@@ -3,21 +3,19 @@
 
 ;; This file is part of Egg on Mule (Multilingal Environment)
 
-;; Egg is distributed in the forms of patches to GNU
-;; Emacs under the terms of the GNU EMACS GENERAL PUBLIC
-;; LICENSE which is distributed along with GNU Emacs by the
-;; Free Software Foundation.
+;; Egg is free software; you can redistribute it and/or modify
+;; it under the terms of the GNU General Public License as published by
+;; the Free Software Foundation; either version 2, or (at your option)
+;; any later version.
 
 ;; Egg is distributed in the hope that it will be useful,
-;; but WITHOUT ANY WARRANTY; without even the implied
-;; warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-;; PURPOSE.  See the GNU EMACS GENERAL PUBLIC LICENSE for
-;; more details.
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
 
-;; You should have received a copy of the GNU EMACS GENERAL
-;; PUBLIC LICENSE along with Nemacs; see the file COPYING.
-;; If not, write to the Free Software Foundation, 675 Mass
-;; Ave, Cambridge, MA 02139, USA.
+;; You should have received a copy of the GNU General Public License
+;; along with GNU Emacs; see the file COPYING.  If not, write to
+;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 ;;;==================================================================
 ;;;
@@ -26,7 +24,7 @@
 ;;;=================================================================== 
 
 ;;;
-;;;$B!V$?$^$4!W$O%M%C%H%o!<%/$+$J4A;zJQ49%5!<%P$rMxMQ$7!"(BNemacs $B$G$NF|K\(B
+;;;$B!V$?$^$4!W$O%M%C%H%o!<%/$+$J4A;zJQ49%5!<%P$rMxMQ$7!"(BMule $B$G$NF|K\(B
 ;;; $B8l4D6-$rDs6!$9$k%7%9%F%`$G$9!#!V$?$^$4!WBh#2HG$G$O(B Wnn V3 $B$*$h$S(B 
 ;;; Wnn V4 $B$N$+$J4A;zJQ49%5!<%P$r;HMQ$7$F$$$^$9!#(B
 ;;;
@@ -36,7 +34,7 @@
 ;;; $B$N6S8+(B $BH~5.;R;a$NL?L>$K0M$k$b$N$G$9!#(Begg $B$O!V$?$^$4!W$N1QLu$G$9!#(B
 
 ;;;
-;;; $B;HMQK!$O(B etc/NEMACS.egg $B$r8+$F2<$5$$!#(B
+;;; $B;HMQK!$O(B info/egg-jp $B$r8+$F2<$5$$!#(B
 ;;;
 
 ;;;
@@ -62,7 +60,6 @@
 ;;;
 ;;; (eval-when (load) (require 'wnn-client))
 ;;;
-(provide 'egg)
 
 (defvar egg-version "3.09" "Version number of this version of Egg. ")
 ;;; Last modified date: Fri Sep 25 12:59:00 1992
@@ -383,6 +380,8 @@
 ;;;;  Aug-25-88 toroku-region$B$GEPO?$9$kJ8;zNs$+$i(Bno graphic character$B$r(B
 ;;;;  $B<+F0E*$K=|$/$3$H$K$7$?!#(B
 
+(eval-when-compile (require 'egg-jsymbol))
+
 ;;;----------------------------------------------------------------------
 ;;;
 ;;; Version control routine
@@ -445,7 +444,7 @@
      its:*current-map*
      mode-line-egg-mode)
    *protected-local-variables*)
-  "List of buffer local variables protected from ""kill-all-local-variables"" ."
+  "List of buffer local variables protected from \"kill-all-local-variables\" ."
   )
 
 ;;;----------------------------------------------------------------------
@@ -598,6 +597,9 @@
 	(minibuffer (window-buffer (minibuffer-window)))
 	value)
     (save-window-excursion
+      (if (fboundp 'redirect-frame-focus)
+	  (redirect-frame-focus (selected-frame)
+				(window-frame (minibuffer-window))))
       (set-window-buffer (minibuffer-window) menubuffer)
       (select-window (minibuffer-window))
       (set-buffer menubuffer)
@@ -654,7 +656,7 @@
 	 )
 	;; end of patch
 	(while (not finished)
-	  (let ((ch (read-char)))
+	  (let ((ch (read-char-exclusive)))
 	    (setq quit-flag nil)
 	    (cond
 	     ((= ch ?\C-a)
@@ -1086,7 +1088,7 @@
     (?. . "$B!%(B")
     (?/ . "$B!?(B")
     (?: . "$B!'(B")
-    (?; . "$B!((B")
+    (?\; . "$B!((B")
     (?< . "$B!c(B")
     (?= . "$B!a(B")
     (?> . "$B!d(B")
@@ -1376,6 +1378,12 @@ INPUT $B$,F~NO$5$l$?;~$K>r7o(B CONDi $B$r=g<!D4$Y!$@.N)$7$?;~$K$O(B OUTPUTi
 $B=PNO$9$k!%(B"
   (list 'its-defrule* input (list 'quote (cons 'cond conds))))
 
+(defmacro its-defrule-conditional* (input state map &rest conds)
+  "(its-defrule-conditional INPUT STATE MAP ((COND1 OUTPUT1) ... (CONDn
+OUTPUTn)))$B$O(B INPUT $B$,F~NO$5$l$?;~$K>uBV(B STATE $B$rI=<($7!$>r7o(B CONDi $B$r(B
+$B=g<!D4$Y!$@.N)$7$?;~$K$O(B OUTPUTi $B$r=PNO$9$k!%(B"
+  (list 'its-defrule* input (list 'quote (cons 'cond conds)) state map))
+
 (defun its-defrule-select-mode-temporally (input name)
   "INPUT $B$,F~NO$5$l$k$H(B temporally-mode $B$H$7$F(B NAME $B$,A*Br$5$l$k!%(B"
 
@@ -1399,9 +1407,9 @@ INPUT $B$,F~NO$5$l$?;~$K>r7o(B CONDi $B$r=g<!D4$Y!$@.N)$7$?;~$K$O(B OUTPUTi
 	  (funcall its:make-terminal-state map input action state)))
 	(if (and its:*defrule-verbose* (map-action map))
 	    (if action
-		(notify "(its-defrule ""%s"" ""%s"" ) $B$r:FDj5A$7$^$7$?!%(B"
+		(notify "(its-defrule \"%s\" \"%s\" ) $B$r:FDj5A$7$^$7$?!%(B"
 			input action)
-	      (notify "(its-defrule ""%s"" ""%s"" )$B$r:o=|$7$^$7$?!%(B"
+	      (notify "(its-defrule \"%s\" \"%s\" )$B$r:o=|$7$^$7$?!%(B"
 		      input (map-action map))))
 	(if (and (null action) (map-terminalp map)) nil
 	  (progn (map-set-action map action)
@@ -1610,7 +1618,7 @@ INPUT $B$,F~NO$5$l$?;~$K>r7o(B CONDi $B$r=g<!D4$Y!$@.N)$7$?;~$K$O(B OUTPUTi
 (defun its:peek-char ()
   (if (= (point) its:*buff-e*)
       (if its:*interactive*
-	  (setq unread-command-char (read-char))
+	  (setq unread-command-events (list (read-char-exclusive)))
 	nil)
     (following-char)))
 
@@ -1619,7 +1627,7 @@ INPUT $B$,F~NO$5$l$?;~$K>r7o(B CONDi $B$r=g<!D4$Y!$@.N)$7$?;~$K$O(B OUTPUTi
       (progn 
 	(setq its:*char-from-buff* nil)
 	(if its:*interactive*
-	    (read-char)
+	    (read-char-exclusive)
 	  nil))
     (let ((ch (following-char)))
       (setq its:*char-from-buff* t)
@@ -1630,26 +1638,26 @@ INPUT $B$,F~NO$5$l$?;~$K>r7o(B CONDi $B$r=g<!D4$Y!$@.N)$7$?;~$K$O(B OUTPUTi
   (if its:*char-from-buff*
       (save-excursion
 	(its:insert-char ch))
-    (if ch (setq unread-command-char ch))))
+    (if ch (setq unread-command-events (list ch)))))
 
 (defun its:insert-char (ch)
   (insert ch))
 
 (defun its:ordinal-charp (ch)
   (and (numberp ch) (<= ch 127)
-       (eq (aref fence-mode-map ch) 'fence-self-insert-command)))
+       (eq (lookup-key fence-mode-map (char-to-string ch)) 'fence-self-insert-command)))
 
 (defun its:delete-charp (ch)
   (and (numberp ch) (<= ch 127)
-       (eq (aref fence-mode-map ch) 'fence-backward-delete-char)))
+       (eq (lookup-key fence-mode-map (char-to-string ch)) 'fence-backward-delete-char)))
     
 (defun fence-self-insert-command ()
   (interactive)
   (cond((or (not egg:*input-mode*)
-	    (null (get-next-map its:*current-map* last-command-char)))
-	(insert last-command-char))
+	    (null (get-next-map its:*current-map* last-command-event)))
+	(insert last-command-event))
        (t
-	(insert last-command-char)
+	(insert last-command-event)
 	(its:translate-region (1- (point)) (point) t))))
 
 ;;;
@@ -1689,7 +1697,7 @@ Arguments are STRING, ALIST and optional PRED. ALIST must be no obarray."
     (if quit-flag
 	(progn
 	  (setq quit-flag nil)
-	  (setq unread-command-char ?\^G)))))
+	  (setq unread-command-events (list ?\^G))))))
 
 (defun car-string-lessp (item1 item2)
   (string-lessp (car item1) (car item2)))
@@ -1810,7 +1818,7 @@ Arguments are STRING, ALIST and optional PRED. ALIST must be no obarray."
 	(ch 0))
     (while (not (= ch ?\^L))
       (insert "<" (nth (car action-output)output) ">")
-      (setq ch (read-char))
+      (setq ch (read-char-exclusive))
       (cond ((= ch ?\^N)
 	     (setcar action-output
 		     (mod (1+ (car action-output)) (length output))))
@@ -1870,7 +1878,7 @@ Arguments are STRING, ALIST and optional PRED. ALIST must be no obarray."
       (setq action (get-action newmap))
 
       (cond
-       ((and its:*interactive* (not its:*char-from-buff*) (= ch ?\^@))
+       ((and its:*interactive* (not its:*char-from-buff*) (numberp ch) (= ch ?\^@))
 	(delete-region its:*buff-s* (point))
 	(let ((i 1))
 	  (while (<= i its:*level*)
@@ -2063,7 +2071,7 @@ Arguments are STRING, ALIST and optional PRED. ALIST must be no obarray."
 
     (set-marker its:*buff-s* nil)
     (set-marker its:*buff-e* nil)
-    (if (and its:*interactive* ch) (setq unread-command-char ch))
+    (if (and its:*interactive* ch) (setq unread-command-events (list ch)))
     ))
 
 ;;;----------------------------------------------------------------------
@@ -2356,7 +2364,7 @@ Arguments are STRING, ALIST and optional PRED. ALIST must be no obarray."
 
 (let ((ch 32))
   (while (< ch 127)
-    (aset global-map ch 'egg-self-insert-command)
+    (define-key global-map (char-to-string ch) 'egg-self-insert-command)
     (setq ch (1+ ch))))
 
 ;;;
@@ -2383,14 +2391,14 @@ Arguments are STRING, ALIST and optional PRED. ALIST must be no obarray."
 ;;;
 
 (defun egg:do-auto-fill ()
-  (if (and auto-fill-hook (not buffer-read-only)
+  (if (and auto-fill-function (not buffer-read-only)
 	   (> (current-column) fill-column))
       (let ((ocolumn (current-column)))
-	(run-hooks 'auto-fill-hook)
+	(funcall auto-fill-function)
 	(while (and (< fill-column (current-column))
 		    (< (current-column) ocolumn))
   	  (setq ocolumn (current-column))
-	  (run-hooks 'auto-fill-hook)))))
+	  (funcall auto-fill-function)))))
 
 ;;;----------------------------------------------------------------------
 ;;;
@@ -2400,34 +2408,45 @@ Arguments are STRING, ALIST and optional PRED. ALIST must be no obarray."
 
 (defconst egg:*fence-open*   "|" "*$B%U%'%s%9$N;OE@$r<($9J8;zNs(B")
 (defconst egg:*fence-close*  "|" "*$B%U%'%s%9$N=*E@$r<($9J8;zNs(B")
-(defconst egg:*fence-attribute* nil  "*$B%U%'%s%9I=<($KMQ$$$k(Battribute $B$^$?$O(B nil")
+(defconst egg:*fence-face* nil  "*$B%U%'%s%9I=<($KMQ$$$k(B face $B$^$?$O(B nil")
+(make-variable-buffer-local
+ (defvar egg:*fence-overlay* nil "$B%U%'%s%9I=<(MQ(B overlay"))
 
-(defvar egg:*attribute-alist*
+(defvar egg:*face-alist*
   '(("nil" . nil)
-    ("inverse" . inverse) ("underline" . underline) ("bold" . bold)))
+    ("highlight" . highlight) ("modeline" . modeline)
+    ("inverse" . modeline) ("underline" . underline) ("bold" . bold)
+    ("region" . region)))
 
-(defun set-egg-fence-mode-format (open close &optional attr)
+(defun set-egg-fence-mode-format (open close &optional face)
   "fence mode $B$NI=<(J}K!$r@_Dj$9$k!#(BOPEN $B$O%U%'%s%9$N;OE@$r<($9J8;zNs$^$?$O(B nil$B!#(B\n\
 CLOSE$B$O%U%'%s%9$N=*E@$r<($9J8;zNs$^$?$O(B nil$B!#(B\n\
-optional ATTR $B$O%U%'%s%96h4V$rI=<($9$kB0@-(B $B$^$?$O(B nil$B!J(Bx11term $B$N$_$GM-8z!K(B"
+$BBh(B3$B0z?t(B FACE $B$,;XDj$5$l$F(B nil $B$G$J$1$l$P!"%U%'%s%96h4V$NI=<($K$=$l$r;H$&!#(B"
   (interactive (list (read-string "$B%U%'%s%93+;OJ8;zNs(B: ")
 		     (read-string "$B%U%'%s%9=*N;J8;zNs(B: ")
-		     (cdr (assoc (completing-read "$B%U%'%s%9I=<(B0@-(B: " egg:*attribute-alist*)
-				 egg:*attribute-alist*))))
+		     (cdr (assoc (completing-read "$B%U%'%s%9I=<(B0@-(B: " egg:*face-alist*)
+				 egg:*face-alist*))))
 
   (if (and (or (stringp open) (null open))
 	   (or (stringp close) (null close))
-	   (egg:member attr '(underline inverse bold nil)))
+	   (or (null face) (memq face (face-list))))
       (progn
 	(setq egg:*fence-open* (or open "")
 	      egg:*fence-close* (or close "")
-	      egg:*fence-attribute* attr)
-	(if attr (require 'attribute))
+	      egg:*fence-face* face)
+	(if (overlayp egg:*fence-overlay*)
+	    (overlay-put egg:*fence-overlay* 'face egg:*fence-face*))
 	t)
-    (error "Wrong type of argument: %s %s %s" open close attr)))
+    (error "Wrong type of argument: %s %s %s" open close face)))
 
-(defconst egg:*region-start* (make-marker))
-(defconst egg:*region-end*   (set-marker-type (make-marker) t))
+;(defconst egg:*region-start* (make-marker))
+;(defconst egg:*region-end*   (set-marker-type (make-marker) t))
+(defvar egg:*region-start* nil)
+(defvar egg:*region-end*   nil)
+(make-variable-buffer-local 'egg:*region-start*)
+(make-variable-buffer-local 'egg:*region-end*)
+(set-default 'egg:*region-start* nil)
+(set-default 'egg:*region-end* nil)
 (defvar egg:*global-map-backup* nil)
 (defvar egg:*local-map-backup*  nil)
 
@@ -2449,7 +2468,7 @@ optional ATTR $B$O%U%'%s%96h4V$rI=<($9$kB0@-(B $B$^$?$O(B nil$B!J(Bx11term
 	   mc-flag
 	   egg:*mode-on* egg:*input-mode* 
 	   (not egg:*in-fence-mode*) ;;; inhibit recursive fence mode
-	   (not (= last-command-char  ?  )))
+	   (not (= last-command-event  ?  )))
       (egg:enter-fence-mode-and-self-insert)
     (progn
       ;; treat continuous 20 self insert as a single undo chunk.
@@ -2468,26 +2487,28 @@ optional ATTR $B$O%U%'%s%96h4V$rI=<($9$kB0@-(B $B$^$?$O(B nil$B!J(Bx11term
 	  (if (<= 1 arg)
 	      (funcall self-insert-after-hook
 		       (- (point) arg) (point)))
-	(if (= last-command-char ? ) (egg:do-auto-fill))))))
+	(if (= last-command-event ? ) (egg:do-auto-fill))))))
 
 
 (defun egg:enter-fence-mode-and-self-insert () 
   (enter-fence-mode)
-  (setq unread-command-char last-command-char))
+  (setq unread-command-events (list last-command-event)))
 
-(defun egg:fence-attribute-on ()
-  (egg:set-region-attribute egg:*fence-attribute* t))
+(defun egg:fence-face-on ()
+  (if egg:*fence-face*
+      (progn
+	(if (overlayp egg:*fence-overlay*)
+	    nil
+	  (setq egg:*fence-overlay* (make-overlay 1 1 nil t))
+	  (if egg:*fence-face* (overlay-put egg:*fence-overlay* 'face egg:*fence-face*)))
+	(move-overlay egg:*fence-overlay* egg:*region-start* egg:*region-end* ) )))
 
-(defun egg:fence-attribute-off ()
-  (egg:set-region-attribute egg:*fence-attribute* nil))
+(defun egg:fence-face-off ()
+  (and egg:*fence-face*
+       (overlayp egg:*fence-overlay*)
+       (delete-overlay egg:*fence-overlay*) ))
 
 (defun enter-fence-mode ()
-  (if overwrite-mode
-      (save-excursion
-	(let ((p (point)))
-	  (end-of-line)
-	  (setq egg:*overwrite-mode-previous-string*
-		(buffer-substring p (point))))))
   ;;;(buffer-flush-undo (current-buffer))
   (and (boundp 'disable-undo) (setq disable-undo t))
   (setq egg:*in-fence-mode* t)
@@ -2498,10 +2519,12 @@ optional ATTR $B$O%U%'%s%96h4V$rI=<($9$kB0@-(B $B$^$?$O(B nil$B!J(Bx11term
   ;;;(use-local-map nil)
   (use-local-map fence-mode-map)
   (insert egg:*fence-open*)
+  (or (markerp egg:*region-start*) (setq egg:*region-start* (make-marker)))
   (set-marker egg:*region-start* (point))
   (insert egg:*fence-close*)
+  (or (markerp egg:*region-end*) (setq egg:*region-end* (set-marker-type (make-marker) t)))
   (set-marker egg:*region-end* egg:*region-start*)
-  (egg:fence-attribute-on)
+  (egg:fence-face-on)
   (goto-char egg:*region-start*)
   )
 
@@ -2516,6 +2539,7 @@ optional ATTR $B$O%U%'%s%96h4V$rI=<($9$kB0@-(B $B$^$?$O(B nil$B!J(Bx11term
 (defun henkan-fence-region ()
   (interactive)
   (setq egg:*henkan-fence-mode* t)
+  (egg:fence-face-off)
   (henkan-region-internal egg:*region-start* egg:*region-end* ))
 
 (defun fence-katakana  ()
@@ -2590,7 +2614,7 @@ optional ATTR $B$O%U%'%s%96h4V$rI=<($9$kB0@-(B $B$^$?$O(B nil$B!J(Bx11term
   (interactive)
   (delete-region (- egg:*region-start* (length egg:*fence-open*)) egg:*region-start*)
   (delete-region egg:*region-end* (+ egg:*region-end* (length egg:*fence-close*)))
-  (egg:fence-attribute-off)
+  (egg:fence-face-off)
   (if its:*previous-map*
       (setq its:*current-map* its:*previous-map*
 	    its:*previous-map* nil))
@@ -2632,22 +2656,35 @@ correspoding to character position.")
   (delete-region egg:*region-start* egg:*region-end*)
   (fence-exit-mode))
 
-(defvar fence-mode-map (make-keymap))
+(defun fence-mode-help-command ()
+  "Display documentation for fence-mode."
+  (interactive)
+  (let ((buf "*Help*"))
+    (if (eq (get-buffer buf) (current-buffer))
+	(henkan-quit)
+      (with-output-to-temp-buffer buf
+	(princ (substitute-command-keys "The keys that are defined for the fence mode here are:\\{fence-mode-map}"))
+	(print-help-return-message)))))
 
+(defvar fence-mode-map (make-keymap))
 (defvar fence-mode-esc-map (make-keymap))
 
+(define-key fence-mode-map [t] 'undefined)
+(define-key fence-mode-esc-map [t] 'undefined)
+
 (let ((ch 0))
   (while (<= ch 127)
-    (aset fence-mode-map ch 'undefined)
-    (aset fence-mode-esc-map ch 'undefined)
+    (define-key fence-mode-map (char-to-string ch) 'undefined)
+    (define-key fence-mode-esc-map (char-to-string ch) 'undefined)
     (setq ch (1+ ch))))
 
 (let ((ch 32))
   (while (< ch 127)
-    (aset fence-mode-map ch 'fence-self-insert-command)
+    (define-key fence-mode-map (char-to-string ch) 'fence-self-insert-command)
     (setq ch (1+ ch))))
 
 (define-key fence-mode-map "\e"   fence-mode-esc-map)
+(define-key fence-mode-map [escape]   fence-mode-esc-map)
 (define-key fence-mode-map "\eh"  'fence-hiragana)
 (define-key fence-mode-map "\ek"  'fence-katakana)
 (define-key fence-mode-map "\e<"  'fence-hankaku)
@@ -2660,6 +2697,7 @@ correspoding to character position.")
 (define-key fence-mode-map "\eZ"    'its:select-zenkaku-upcase)
 (define-key fence-mode-map " "    'henkan-fence-region-or-single-space)
 (define-key fence-mode-map "\C-@" 'henkan-fence-region)
+(define-key fence-mode-map [?\C-\ ] 'henkan-fence-region)
 (define-key fence-mode-map "\C-a" 'fence-beginning-of-line)
 (define-key fence-mode-map "\C-b" 'fence-backward-char)
 (define-key fence-mode-map "\C-c" 'fence-cancel-input)
@@ -2667,28 +2705,23 @@ correspoding to character position.")
 (define-key fence-mode-map "\C-e" 'fence-end-of-line)
 (define-key fence-mode-map "\C-f" 'fence-forward-char)
 (define-key fence-mode-map "\C-g" 'fence-cancel-input)
-(define-key fence-mode-map "\C-h" 'help-command)
-(define-key fence-mode-map "\C-i" 'undefined)  
-(define-key fence-mode-map "\C-j" 'undefined)  ;;; LFD
+(define-key fence-mode-map "\C-h" 'fence-mode-help-command)
 (define-key fence-mode-map "\C-k" 'fence-kill-line)
 (define-key fence-mode-map "\C-l" 'fence-exit-mode)
 (define-key fence-mode-map "\C-m" 'fence-exit-mode)  ;;; RET
-(define-key fence-mode-map "\C-n" 'undefined)
-(define-key fence-mode-map "\C-o" 'undefined)
-(define-key fence-mode-map "\C-p" 'undefined)
+(define-key fence-mode-map [return] 'fence-exit-mode)
 (define-key fence-mode-map "\C-q" 'its:select-previous-mode)
-(define-key fence-mode-map "\C-r" 'undefined)
-(define-key fence-mode-map "\C-s" 'undefined)
 (define-key fence-mode-map "\C-t" 'fence-transpose-chars)
-(define-key fence-mode-map "\C-u" 'undefined)
-(define-key fence-mode-map "\C-v" 'undefined)
 (define-key fence-mode-map "\C-w" 'henkan-fence-region)
-(define-key fence-mode-map "\C-x" 'undefined)
-(define-key fence-mode-map "\C-y" 'undefined)
 (define-key fence-mode-map "\C-z" 'eval-expression)
-(define-key fence-mode-map "\C-|" 'fence-toggle-egg-mode)
+(define-key fence-mode-map "\C-\\" 'fence-toggle-egg-mode)
 (define-key fence-mode-map "\C-_" 'jis-code-input)
 (define-key fence-mode-map "\177" 'fence-backward-delete-char)
+(define-key fence-mode-map [delete] 'fence-backward-delete-char)
+(define-key fence-mode-map [backspace] 'fence-backward-delete-char)
+(define-key fence-mode-map [right] 'fence-forward-char)
+(define-key fence-mode-map [left] 'fence-backward-char)
+
 
 ;;;----------------------------------------------------------------------
 ;;;
@@ -2750,4 +2783,6 @@ correspoding to character position.")
 (autoload 'busyu-input "busyu" nil t)	;92.10.18 by K.Handa
 (autoload 'kakusuu-input "busyu" nil t)	;92.10.18 by K.Handa
 
-;;; End of egg.el
+(provide 'egg)
+
+;;; egg.el ends here
