@@ -32,33 +32,34 @@
 (its-defrule-select-mode-temporally "q" "downcase")
 (its-defrule-select-mode-temporally "Q" "zenkaku-downcase")
 
+;;; $B!V$C!W$NF~NO(B
+
 (dolist (aa '("k" "s" "t" "h" "y" "r" "w" "g" "z" "d" "b"
 		 "p" "c" "f" "j" "v"))
   (its-defrule (concat aa aa) "$B$C(B" aa))
 
 (its-defrule "tch"  "$B$C(B" "ch")
 
+;;; $B!V$s!W$NF~NO(B
+
 (dolist (q1 '("b" "m" "p"))
   (its-defrule (concat "m" q1) "$B$s(B" q1))
 
 (its-defrule "N" "$B$s(B")
 
-(defvar enable-double-n-syntax nil "*""nn""$B$r(B""$B$s(B""$B$KJQ49$9$k(B")
-
-(its-defrule-conditional "n" 
-  ((not enable-double-n-syntax) "$B$s(B")
-  (t nil))
-
-(its-defrule-conditional "nn" 
-  (enable-double-n-syntax "$B$s(B")
-  (t nil))
-
 (its-defrule "n'" "$B$s(B")
 
+(defvar enable-double-n-syntax nil "*""nn""$B$r(B""$B$s(B""$B$KJQ49$9$k(B")
+
+(its-defrule "n" "$B$s(B")
+(its-defrule-conditional* "nn" "$B$s(B" nil
+			  (enable-double-n-syntax "$B$s(B")
+			  (t nil))
+
 ;; 92.7.6 by Y.Kawabe
-(dolist (aa '("k" "s" "t" "c" "h" "f" "m" "y" "r" "l"
-	      "w" "g" "z" "j" "d" "b" "v" "p" "x"))
-  (its-defrule (concat "n" aa) "$B$s(B" aa))
+;;(dolist (aa '("k" "s" "t" "c" "h" "f" "m" "y" "r" "l"
+;;	      "w" "g" "z" "j" "d" "b" "v" "p" "x"))
+;;  (its-defrule (concat "n" aa) "$B$s(B" aa))
 ;; end of patch
 
 (let ((small '"x" ))
@@ -311,17 +312,17 @@
 (its-defrule   "?"   "$B!)(B")
 (its-defrule   "/"   "$B!?(B")
 
-(defvar use-kuten-for-comma t   "*$B%3%s%^$r6gE@$KJQ49$9$k(B")
-(defvar use-touten-for-period t "*$B%T%j%*%I$rFIE@$KJQ49$9$k(B")
-
-(its-defrule-conditional ","
-  (use-kuten-for-comma "$B!"(B")
-  (t "$B!$(B"))
+(defvar use-kuten-for-period t "*$B%T%j%*%I$r6gE@$KJQ49$9$k(B")
+(defvar use-touten-for-comma t "*$B%3%s%^$rFIE@$KJQ49$9$k(B")
 
 (its-defrule-conditional "."
-  (use-touten-for-period "$B!#(B")
+  (use-kuten-for-period "$B!#(B")
   (t "$B!%(B"))
 
+(its-defrule-conditional ","
+  (use-touten-for-comma "$B!"(B")
+  (t "$B!$(B"))
+
 ;;; Escape character to Zenkaku inputs
 
 (defvar zenkaku-escape "Z")
