@@ -297,17 +297,17 @@
 (its-defrule   "?"   "$B!)(B")
 (its-defrule   "/"   "$B!?(B")
 
-(defvar use-kuten-for-comma t   "*$B%3%s%^$r6gE@$KJQ49$9$k(B")
-(defvar use-touten-for-period t "*$B%T%j%*%I$rFIE@$KJQ49$9$k(B")
-
-(its-defrule-conditional ","
-  (use-kuten-for-comma "$B!"(B")
-  (t "$B!$(B"))
+(defvar use-kuten-for-period t "*$B%T%j%*%I$r6gE@$KJQ49$9$k(B")
+(defvar use-touten-for-comma t "*$B%3%s%^$rFIE@$KJQ49$9$k(B")
 
 (its-defrule-conditional "."
-  (use-touten-for-period "$B!#(B")
+  (use-kuten-for-period "$B!#(B")
   (t "$B!%(B"))
 
+(its-defrule-conditional ","
+  (use-touten-for-comma "$B!"(B")
+  (t "$B!$(B"))
+
 ;;; Escape character to Zenkaku inputs
 
 (defvar zenkaku-escape "Z")
