@@ -87,6 +87,8 @@
 ;;; 93.7.17  modified for Mule Ver.0.9.8 by K.Hirokawa <hirokawa@rics.co.jp>
 ;;;	find-process-coding-system-rsh and find-process-coding-system-env
 ;;;	should call find-process-coding-system by 'apply'.
+;;; 94.2.23  modified for Mule Ver.1.1 by K.Handa <handa@etl.go.jp>
+;;;	Big change of coding-system handling.
 
 ;;;####### Here's the history before Ver.0.9.5
 ;;; Ver.2.05 92.4.30 modified for Mule Ver.0.9.4
@@ -190,13 +192,10 @@
 (defun toggle-default-mc-flag ()
   "Toggle default-mc-flag."
   (interactive)		;;; patch by H.Nakahara 87.6.24
-  (setq-default mc-flag (not (default-value 'mc-flag)))
-  (if mc-flag
-      (goto-char (point)))
-  (if (interactive-p) (redraw-display)))
+  (setq default-mc-flag (null default-mc-flag)))
 
 (defun set-default-file-coding-system (coding-system)
-  "Set default-file-coding-system to CODING-SYSTEM,
+  "Set default value of file-coding-system to CODING-SYSTEM,
 which should be a lisp object created by make-coding-system."
   (interactive "zDefault file-coding-system: ")
   (setq-default file-coding-system coding-system)
@@ -210,223 +209,37 @@ INPUT and OUTPUT, which should be lisp objects created by make-coding-system."
   (setq default-process-coding-system (cons input output))
   (update-mode-lines))
 
-(defun charset-doc (lc)
-  (if lc (nth 6 (character-set lc)) "none" )) ;92.12.31 by T.Shingu
-
-;; 93.2.28 by K.Handa
-(defun describe-designation (flags graphic)
-  (let ((lc (aref flags graphic))
-	lc1)
-    (if (integerp lc) (setq lc1 (if (> lc 0) lc (- lc))))
-    (princ (format "  G%d -- %s"
-		   graphic
-		   (or (and lc1 (charset-doc lc1))
-		       (and (eq lc lc-invalid) "never used")
-		       "none")))
-    (princ (if (and lc1 (< lc 0))
-	       " (explicit designation required)\n"
-	     "\n"))))
-;; end of patch
-
-(defun describe-coding-system (code)
-  "Display documentation of the coding-system CODE."
-  (interactive "zCoding-system: ")
-  (check-coding-system code)
-  (with-output-to-temp-buffer "*Help*"
-    (princ "Coding-system ")
-    (princ code)
-    (princ " [")
-    (princ (char-to-string (get-code-mnemonic code)))
-    (princ "]: \n")
-    (if (not code) nil
-      (princ "  ")
-      (princ (get-code-document code))
-      (princ "\nType: ")
-      (let ((type (get-code-type code)) flags)
-	(princ type)
-	(princ " [")
-	(cond ((eq type t)
-	       (princ "AUTOCONV]\n"))
-	      ((eq type 0)
-	       (princ "INTERNAL]\n"))
-	      ((eq type 1)
-	       (princ "Shift-JIS]\n"))
-	      ((eq type 2)
-	       (princ "ISO-2022]\n")
-	       (setq flags (get-code-flags code))
-	       (princ "Initial designations:\n")
-	       ;; 93.2.28 by K.Handa
-	       (describe-designation flags 0)
-	       (describe-designation flags 1)
-	       (describe-designation flags 2)
-	       (describe-designation flags 3)
-	       ;; end of patch
-	       (princ "Other Form: \n")
-	       (princ (if (aref flags 4) "ShortForm" "LongForm"))
-	       (if (aref flags 5) (princ ", ASCII@EOL"))
-	       (if (aref flags 6) (princ ", ASCII@CNTL"))
-	       (princ (if (aref flags 7) ", 7bit" ", 8bit"))
-	       (if (aref flags 8) (princ ", UseLockingShift"))
-	       (if (aref flags 9) (princ ", UseRoman"))
-	       (if (aref flags 10) (princ ", UseOldJIS"))
-	       (princ ".\n"))
-	      ((eq type 3)
-	       (princ "Big5]\n  ")
-	       (princ (if flags "Big-ETen\n" "Big-HKU\n")))
-	      ((eq type 4)
-	       (princ "private]\n  Conversion routine written in CCL."))
-	      (t (princ "unknown\]\n")))))))
-
-(defun list-coding-system-briefly ()
-  "Display coding-systems currently used with a brief format in mini-buffer."
-  (interactive)
-					; 92.5.18 by M.Higashida
-  (let ((cs (and (fboundp 'process-coding-system) (process-coding-system))))
-    (message
-     "current: [FKDPp=%c%c%c%c%c] default: [FPp=%c%c%c]"
-     (get-code-mnemonic file-coding-system)
-     (get-code-mnemonic keyboard-coding-system)
-     (get-code-mnemonic display-coding-system)
-     (get-code-mnemonic (car cs))
-     (get-code-mnemonic (cdr cs))
-     (get-code-mnemonic (default-value 'file-coding-system))
-     (get-code-mnemonic (car default-process-coding-system))
-     (get-code-mnemonic (cdr default-process-coding-system)))))
-
-(defun princ-coding-system (code)
-  (princ ": ")
-  (princ code)
-  (princ " [")
-  (princ (char-to-string (get-code-mnemonic code)))
-  (princ "]\n"))
-
-;; 92.6.26 by M.Shikida -- Spell in comment corrected.
-(defun list-coding-system ()
-  "Describe coding-systems currently used with a detail format"
-  (interactive)
-					; 92.5.18 by M.Higashida
-  (let ((cs (and (fboundp 'process-coding-system) (process-coding-system))))
-    (with-output-to-temp-buffer "*Help*"
-      (princ "Current:\n  file-coding-system")
-      (princ-coding-system file-coding-system)
-      (princ "  keyboard-coding-system")
-      (princ-coding-system keyboard-coding-system)
-      (princ "  display-coding-system")
-      (princ-coding-system display-coding-system)
-      (if cs
-	  (progn (princ "  process-coding-system (input)")
-		 (princ-coding-system (car cs))
-		 (princ "  process-coding-system (output)")
-		 (princ-coding-system (cdr cs))))
-      (princ "Default:\n  file-coding-system")
-      (princ-coding-system (default-value 'file-coding-system))
-      (princ "  process-coding-system (input)")
-      (princ-coding-system (car default-process-coding-system))
-      (princ "  process-coding-system (output)")
-      (princ-coding-system (cdr default-process-coding-system))
-      (princ "Others:\n  file-coding-system-for-read")
-      (princ-coding-system file-coding-system-for-read)
-      )))
-
 ;; 93.6.9 by K.Handa
 (defun toggle-display-direction ()
   (interactive)
   (setq display-direction (not display-direction))
   (if (interactive-p) (redraw-display)))
-
-;; 92.4.3, 92.9.29 by K.Handa
-(defun detect-code-category (start end &optional highest)
-  "Detect the category of coding-system of the text between START and END.
-Returns a list of symbols of all possible categories.
-In the list, a category of higher priority comes first.
-If only ASCII codes, return t (LF at eol).
-If any inconsistency is found, returns nil.
-If optional third arg HIGHEST is non-nil,
- returns only the category of highest priority."
-  (let ((bits (code-detect-region start end highest))
-	(prio code-priority)
-	code codes)
-    (if (/= (logand bits 256) 0)
-	(setq bits (logand bits 63)))
-    (if (= bits 63)
-	t
-      (while prio
-	(if (/= (logand bits (get (car prio) 'code-priority-value)) 0)
-	    (setq code (symbol-value (car prio))
-		  codes (cons code codes)))
-	(setq prio (cdr prio)))
-      (setq codes (nreverse codes))
-      (if highest (car codes) codes))))
+;; end of patch
 
 (defun what-coding-system (start end &optional arg)
   "Show coding-system of text in the region.
 With prefix arg, show all possible coding systems.
-From program, use detect-code-category instead."
+From program, use code-detect-region instead."
   (interactive "r\nP")
-  (let ((code (detect-code-category start end (null arg))))
-    (if (listp code)
-	(message "%s" (mapconcat 'symbol-name code " "))
-      (message "%s" code))))
-;; end of patch
-
-;; 92.10.12 by K.Handa
-(defun set-x-default-font (lc font encoding)
-  "Set FONTNAME (2nd arg) ane ENCODING (3rd arg) as a default
-for the character set LEADING-CHAR (1st arg)."
-  (if (boundp 'x-default-fonts)
-      (progn
-	(setq lc (logand lc 127))
-	(aset x-default-fonts lc font)
-	(aset x-default-encoding lc encoding))))
-
-(defun list-character-sets ()
-  "Display a list of existing character sets."
-  (interactive)
-  (with-output-to-temp-buffer "*Help*"
-    (princ "## LIST OF CHARACTER SETS\n")
-    (princ "## LC:BYTES:COLUMNS:TYPE:GRAPHIC:FINAL-CHAR:DIRECTION:DOC\n")
-    (let (lc charset)
-      (setq lc 0)
-      (while (< lc 256)
-	(setq charset (character-set lc))
-	(if charset
-	    ;; 93.5.7 by K.Handa
-	    (princ (format "%03d:%d:%d:%d:%d:%d:%d:%s\n" lc
-			   (nth 0 charset) (nth 1 charset) (nth 2 charset)
-			   (nth 3 charset) (nth 4 charset) (nth 5 charset)
-			   (nth 6 charset))))
-	(setq lc (logior (1+ lc) 128))))))
-;; end of patch
+  (let ((codings (code-detect-region start end)))
+    (message "%s" (if arg codings (car codings)))))
 
 (define-key help-map "T" 'help-with-tutorial-for-mule)
-(define-key help-map "N" 'view-mule-news)
 
-(defun help-with-tutorial-for-mule ()
+(defvar help-with-tutorial-language-alist
+  '(("Japanese" . ".jp")
+    ("Korean" . ".kr")
+    ("Thai" . ".th")))
+
+(defun help-with-tutorial-for-mule (language)
   "Select the Mule learn-by-doing tutorial."
-  (interactive)
-  (let ((file (expand-file-name "~/MULE.tut")))
-    (delete-other-windows)
-    (if (get-file-buffer file)
-	(switch-to-buffer (get-file-buffer file))
-      (switch-to-buffer (create-file-buffer "~/MULE.tut"))
-      (setq buffer-file-name file)
-      (setq default-directory (expand-file-name "~/"))
-      (setq auto-save-file-name nil)
-      (insert-file-contents (expand-file-name "MULE.tut" exec-directory))
-      (goto-char (point-min))
-      (search-forward "\n<<")
-      (beginning-of-line)
-      (delete-region (point) (progn (end-of-line) (point)))
-      (newline (- (window-height (selected-window))
-		  (count-lines (point-min) (point))
-		  4))
-      (goto-char (point-min))
-      (set-buffer-modified-p nil))))
-
-(defun view-mule-news ()
-  "Display info on recent changes to Mule." ;; 92.3.25 by M.Kuwada
-  (interactive)
-  (find-file-read-only (expand-file-name "MULE.nws" exec-directory)))
+  (interactive (list (let ((completion-ignore-case t)
+			   lang)
+		       (completing-read
+			"Language: "
+			help-with-tutorial-language-alist))))
+  (setq language (cdr (assoc language help-with-tutorial-language-alist)))
+  (help-with-tutorial (concat "TUTORIAL" (or language ""))))
 
 ;;;
 ;;; Utility functions for Mule
@@ -497,12 +310,23 @@ A prefix argument enables user to specify the coding-system interactively."
 (or (fboundp 'si:find-alternate-file)
     (fset 'si:find-alternate-file (symbol-function 'find-alternate-file)))
 
+;; 94.10.25 by M.Yamamoto <mituharu@is.s.u-tokyo.ac.jp>
 (defun find-alternate-file (filename &optional user-coding-system)
   "Find file FILENAME, select its buffer, kill previous buffer.
 If the current buffer now contains an empty file that you just visited
 \(presumably by mistake), use this command to visit the file you really want.
 A prefix argument enables user to specify the coding-system interactively."
-  (interactive "FFind alternate file: \nZCoding-system: ")
+  (interactive
+   (let ((file buffer-file-name)
+	 (file-name nil)
+	 (file-dir nil))
+     (and file
+	  (setq file-name (file-name-nondirectory file)
+		file-dir (file-name-directory file)))
+     (list (read-file-name
+	    "Find alternate file: " file-dir nil nil file-name)
+	   (if current-prefix-arg
+	       (read-coding-system "Coding-system: ")))))
   (if user-coding-system
       (let ((file-coding-system-for-read
 	     (check-coding-system user-coding-system)))
@@ -587,8 +411,8 @@ A prefix argument enables user to specify the coding-system interactively."
 	  (if (re-search-forward "^From" nil 'move)
 	      (beginning-of-line))
 	  (setq end (point))
-	  ;; 92.5.20 by K.Handa
-	  (code-convert-region start end *auto* *internal*))))))
+	  ;; 92.5.20, 94.2.22 by K.Handa
+	  (code-convert-region start end *autoconv* *internal*))))))
 
 ;;;
 ;;; hack from files.el : hack-local-variables
@@ -659,11 +483,24 @@ for current buffer."
 (or (fboundp 'si:write-file)
     (fset 'si:write-file (symbol-function 'write-file)))
 
+;; 94.10.25 by M.Yamamoto <mituharu@is.s.u-tokyo.ac.jp>
 (defun write-file (filename &optional user-coding-system)
   "Write current buffer into file FILENAME.
 Makes buffer visit that file, and marks it not modified.
+If the buffer is already visiting a file, you can specify
+a directory name as FILENAME, to write a file of the same
+old name in that directory.
 A prefix argument enables user to specify the coding-system interactively."
-  (interactive "FWrite file: \nZCoding-system: ")
+  (interactive
+   (list (if buffer-file-name
+	     (read-file-name "Write file: "
+				 nil nil nil nil)
+	   (read-file-name "Write file: "
+			       (cdr (assq 'default-directory
+					  (buffer-local-variables)))
+			       nil nil (buffer-name)))
+	 (if current-prefix-arg
+	     (read-coding-system "Coding-system: "))))
   (if user-coding-system
       (progn
 	(let ((file-coding-system (check-coding-system user-coding-system)))
@@ -932,7 +769,7 @@ On successful conversion, returns the length of converted text,
 On successful converion, returns the result string,
  else returns nil."
   (let ((curbuf (current-buffer))
-	(tempbuf (get-buffer-create " *code-convert-work*"))
+	(tempbuf (generate-new-buffer " *code-convert-work*"))
 	result)
     (unwind-protect
 	(progn
@@ -942,7 +779,8 @@ On successful converion, returns the result string,
 	  (setq result (code-convert (point-min) (point-max) source target)))
       (if result
 	  (setq result (buffer-string)))
-      (set-buffer curbuf))
+      (set-buffer curbuf)
+      (kill-buffer tempbuf))
     result))
 
 ;; 92.7.31, 92.8.5 by K.Handa
@@ -1017,7 +855,7 @@ If each element of LIST is not a string, it is converted to string
 	(if (= ch 160)
 	    (progn
 	      (setq i (1+ i))
-	      (setq ch (- (aref str i) ch))
+	      (setq ch (- (aref str i) 128))
 	      (aset buf j ch)
 	      (setq i (1+ i) j (1+ j)))
 	  (setq ch (- ch 32))
@@ -1103,3 +941,33 @@ If each element of LIST is not a string, it is converted to string
 	(setq re-private (concat re-private (char-to-string lc))))
       (setq lclist (cons lc lclist)))
     lclist))
+
+;; regexp-word-compile returns:
+;;	((forward . backward)
+;;	 (split1-forward . split1-backward)
+;;	 ...)
+;; define-word-pattern accepts:
+;;	(forward backard split1-backward split1-forward ...)
+(defun set-word-regexp (pattern)
+  "One arg RE should be a compiled pattern created by 'regexp-word-compile."
+  (let (l)
+    (setq l (list (cdr (car pattern)) (car (car pattern))))
+    (setq pattern (cdr pattern))
+    (while pattern
+      (setq l (cons (cdr (car pattern)) l))
+      (setq l (cons (car (car pattern)) l))
+      (setq pattern (cdr pattern)))
+    (define-word-pattern (nreverse l))))
+
+;;; PRIMARY ENVIRONMENT
+(defvar primary-environment nil)
+
+(defun set-primary-environment (env)
+  "Set ENVIRONMENT (e.g. 'japanese, 'chinese) as the primary one."
+  (let ((func (get env 'primary-function)))
+    (if (fboundp func)
+	(progn
+	  (or (featurep env)
+	      (load (symbol-name env) t))
+	  (funcall func)
+	  (setq primary-environment env)))))
