@@ -1,12 +1,15 @@
-;; "Picture mode" -- editing using quarter-plane screen model.
-;; Copyright (C) 1985 Free Software Foundation, Inc.
-;; Principal author K. Shane Hartman
+;;; picture.el --- "Picture mode" -- editing using quarter-plane screen model.
+
+;; Copyright (C) 1985, 1994 Free Software Foundation, Inc.
+
+;; Author: K. Shane Hartman
+;; Maintainer: FSF
 
 ;; This file is part of GNU Emacs.
 
 ;; GNU Emacs is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 1, or (at your option)
+;; the Free Software Foundation; either version 2, or (at your option)
 ;; any later version.
 
 ;; GNU Emacs is distributed in the hope that it will be useful,
@@ -18,25 +21,20 @@
 ;; along with GNU Emacs; see the file COPYING.  If not, write to
 ;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
-;;; 89.3.10  modified for Nemacs Ver.3.0 by K.Handa
-;;; 89.9.7   modified for Nemacs Ver.3.2 by K.Handa
-;;; 91.11.22 modified for Nemacs Ver.4.0.0 by K.Handa <handa@etl.go.jp>
-;;; 92.3.6   modified for Mule Ver.0.9.0 by K.Handa <handa@etl.go.jp>
-;;; 92.7.10  modified for Mule Ver.0.9.5 by K.Handa <handa@etl.go.jp>
-;;;	Bug of horizontal left move in picture-insert-string fixed.
-;;; 92.9.8   modified for Mule Ver.0.9.6 by T.Shingu <shingu@cpr.canon.co.jp>
-;;;	picture-insert-string does nothing if string is NULL.
+;;; Commentary:
 
-(provide 'picture)
+;; This code provides the picture-mode commands documented in the Emacs 
+;; manual.  The screen is treated as a semi-infinite quarter-plane with
+;; support for rectangle operations and `etch-a-sketch' character
+;; insertion in any of eight directions.
 
-;; 89.9.7, 91.11.25 by K.Handa
-(defun move-to-column-force (column &optional force)
+;;; Code:
+
+(defun move-to-column-force (column)
   "Move to column COLUMN in current line.
-Differs from move-to-column in that it creates or modifies whitespace
-if necessary to attain exactly the specified column.
-If optional arg FORCE is non nil and the goal column is occupied by
- multi-column character, the character is replaced by spaces
- to locate the cursor at the exact column."
+Differs from `move-to-column' in that it creates or modifies whitespace
+if necessary to attain exactly the specified column."
+  (or (natnump column) (setq column 0))
   (move-to-column column)
   (let ((col (current-column)))
     (if (< col column)
@@ -45,12 +43,24 @@ If optional arg FORCE is non nil and the goal column is occupied by
 	       (= (preceding-char) ?\t))
 	  (let (indent-tabs-mode)
 	    (delete-char -1)
-	    (indent-to col)
-	    (move-to-column column))))))
+            (indent-to col)
+            (move-to-column column))))
+    ;; This call will go away when Emacs gets real horizontal autoscrolling
+    (hscroll-point-visible)))
 
 
 ;; Picture Movement Commands
 
+(defun picture-beginning-of-line (&optional arg)
+  "Position point at the beginning of the line.
+With ARG not nil, move forward ARG - 1 lines first.
+If scan reaches end of buffer, stop there without error."
+  (interactive "P")
+  (if arg (forward-line (1- (prefix-numeric-value arg))))
+  (beginning-of-line)
+  ;; This call will go away when Emacs gets real horizontal autoscrolling
+  (hscroll-point-visible))
+
 (defun picture-end-of-line (&optional arg)
   "Position point after last non-blank character on current line.
 With ARG not nil, move forward ARG - 1 lines first.
@@ -58,22 +68,27 @@ If scan reaches end of buffer, stop there without error."
   (interactive "P")
   (if arg (forward-line (1- (prefix-numeric-value arg))))
   (beginning-of-line)
-  (skip-chars-backward " \t" (prog1 (point) (end-of-line))))
+  (skip-chars-backward " \t" (prog1 (point) (end-of-line)))
+  ;; This call will go away when Emacs gets real horizontal autoscrolling
+  (hscroll-point-visible))
 
-(defun picture-forward-column (arg &optional force) ; 89.9.7  by K.Handa
+(defun picture-forward-column (arg)
   "Move cursor right, making whitespace if necessary.
 With argument, move that many columns."
   (interactive "p")
-  (move-to-column-force (+ (current-column) arg) force))
+  (let ((target-column (+ (current-column) arg)))
+    (move-to-column-force target-column)
+    ;; Picture mode isn't really suited to multi-column characters,
+    ;; but we might as well let the user move across them.
+    (and (< arg 0)
+	 (> (current-column) target-column)
+	 (forward-char -1))))
 
 (defun picture-backward-column (arg)
   "Move cursor left, making whitespace if necessary.
 With argument, move that many columns."
   (interactive "p")
-  (let ((clm (current-column)))		; 92.3.13 by K.Handa
-    (move-to-column-force (- (current-column) arg))
-    (if (= clm (current-column))
-	(forward-char -1))))
+  (picture-forward-column (- arg)))
 
 (defun picture-move-down (arg)
   "Move vertically down, making whitespace if necessary.
@@ -115,28 +130,6 @@ With argument, move that many lines."
   (interactive)
   (picture-set-motion 1 0))
 
-;;; 89.9.7  by K.Handa
-(defun picture-movement-wnw ()
-  "Move up and left and left after self-inserting character in Picture mode."
-  (interactive)
-  (picture-set-motion -1 -2))
-
-(defun picture-movement-ene ()
-  "Move up and right and right after self-inserting character in Picture mode."
-  (interactive)
-  (picture-set-motion -1 2))
-
-(defun picture-movement-wsw ()
-  "Move down and left and left after self-inserting character in Picture mode."
-  (interactive)
-  (picture-set-motion 1 -2))
-
-(defun picture-movement-ese ()
-  "Move down and right and right after self-inserting character in Picture mode."
-  (interactive)
-  (picture-set-motion 1 2))
-;;; end of patch
-
 (defun picture-movement-nw ()
   "Move up and left after self-inserting character in Picture mode."
   (interactive)
@@ -164,18 +157,17 @@ The mode line is updated to reflect the current direction."
 	picture-horizontal-step horiz)
   (setq mode-name
 	(format "Picture:%s"
-		(car (nthcdr (+ 2 (% horiz 3) (* 5 (1+ (% vert 2))))
-			     '(wnw nw up ne ene Left left none right Right
-				   wsw sw down se ese)))))  ; 89.9.7  by K.Handa
+		(car (nthcdr (+ 1 (% horiz 2) (* 3 (1+ (% vert 2))))
+			     '(nw up ne left none right sw down se)))))
   ;; Kludge - force the mode line to be updated.  Is there a better
   ;; way to this?
   (set-buffer-modified-p (buffer-modified-p))
   (message ""))
 
-(defun picture-move (&optional force)	; 89.9.7 by K.Handa
-  "Move in direction of  picture-vertical-step  and  picture-horizontal-step."
+(defun picture-move ()
+  "Move in direction of `picture-vertical-step' and `picture-horizontal-step'."
   (picture-move-down picture-vertical-step)
-  (picture-forward-column picture-horizontal-step force))
+  (picture-forward-column picture-horizontal-step))
 
 (defun picture-motion (arg)
   "Move point in direction of current picture motion in Picture mode.
@@ -190,7 +182,7 @@ Do \\[command-apropos]  picture-movement  to see commands which control motion."
   "Move point in direction opposite of current picture motion in Picture mode.
 With ARG do it that many times.  Useful for delineating rectangles in
 conjunction with diagonal picture motion.
-Do \\[command-apropos]  picture-movement  to see commands which control motion."
+Do \\[command-apropos] `picture-movement' to see commands which control motion."
   (interactive "p")
   (picture-motion (- arg)))
 
@@ -200,48 +192,16 @@ Do \\[command-apropos]  picture-movement  to see commands which control motion."
 (defun picture-self-insert (arg)
   "Insert this character in place of character previously at the cursor.
 The cursor then moves in the direction you previously specified
-with the commands picture-movement-right, picture-movement-up, etc.
-Do \\[command-apropos]  picture-movement  to see those commands."
+with the commands `picture-movement-right', `picture-movement-up', etc.
+Do \\[command-apropos] `picture-movement' to see those commands."
   (interactive "p")
-  (while (> arg 0)			; 89.9.7 by K.Handa
-    (picture-insert-string (make-string arg last-input-char))))
-
-;;; 89.9.7  by K.Handa
-(defun picture-insert-after-hook (begin end)
-  "Called as a hook after a string is inserted into the current buffer.
-Two arguments BEGIN and END point the head and tail of just inserted string."
-  (let ((str (buffer-substring begin end)))
-    (delete-region begin end)
-    (picture-insert-string str)))
-
-(defun picture-insert-string (str)	; 91.11.25, 92.7.10 by K.Handa
-  "Insert characters in STR one by one in place of character previously
-at the cursor while moving cursor in the direction you previously specified
-with the commands picture-movement-right, picture-movement-up, etc.
-Do \\[command-apropos]  picture-movement  to see those commands."
-  (interactive "sInsert string: ")
-  (let  ((len (length str)))
-    (if (> len 0)
-	(let* ((idx 0)
-	       (c (sref str idx))
-	       (width (char-width c))
-	       clm)
-	  (while idx				; Set to nil at the end of loop
-	    (setq clm (current-column))
-	    (delete-text-in-column clm (+ clm width))
-	    (insert c)
-	    (forward-char -1)
-	    (setq idx (+ idx (char-bytes c)))
-	    (if (< idx len)
-		(setq c (sref str idx) width (char-width c))
-	      (setq idx nil))
-	    (if (/= picture-vertical-step 0)
-		(picture-move 'force)
-	      (if (> picture-horizontal-step 0)
-		  (forward-char 1)
-		(move-to-column-force (- clm width) 'force)))
-	    )))))
-;;; end of patch
+  (while (> arg 0)
+    (setq arg (1- arg))
+    (move-to-column-force (1+ (current-column)))
+    (delete-char -1)
+    (insert last-input-char)
+    (forward-char -1)
+    (picture-move)))
 
 (defun picture-clear-column (arg)
   "Clear out ARG columns after point without moving."
@@ -261,9 +221,9 @@ Do \\[command-apropos]  picture-movement  to see those commands."
 
 (defun picture-clear-line (arg)
   "Clear out rest of line; if at end of line, advance to next line.
-Cleared-out line text goes into the kill ring, as do
-newlines that are advanced over.
-With argument, clear out (and save in kill ring) that many lines."
+Cleared-out line text goes into the kill ring, as do newlines that are
+advanced over.  With argument, clear out (and save in kill ring) that
+many lines."
   (interactive "P")
   (if arg
       (progn
@@ -284,7 +244,9 @@ always moves to the beginning of a line."
     (while (> arg 0)
       (end-of-line)
       (if (eobp) (newline) (forward-char 1))
-      (setq arg (1- arg)))))
+      (setq arg (1- arg))))
+  ;; This call will go away when Emacs gets real horizontal autoscrolling
+  (hscroll-point-visible))
 
 (defun picture-open-line (arg)
   "Insert an empty line after the current line.
@@ -292,7 +254,9 @@ With positive argument insert that many lines."
   (interactive "p")
   (save-excursion
    (end-of-line)
-   (open-line arg)))
+   (open-line arg))
+  ;; This call will go away when Emacs gets real horizontal autoscrolling
+  (hscroll-point-visible))
 
 (defun picture-duplicate-line ()
   "Insert a duplicate of the current line, below it."
@@ -305,6 +269,28 @@ With positive argument insert that many lines."
      (forward-line -1)
      (insert contents))))
 
+;; Like replace-match, but overwrites.
+(defun picture-replace-match (newtext fixedcase literal)
+  (let (ocolumn change pos)
+    (goto-char (setq pos (match-end 0)))
+    (setq ocolumn (current-column))
+    ;; Make the replacement and undo it, to see how it changes the length.
+    (let ((buffer-undo-list nil)
+	  list1)
+      (replace-match newtext fixedcase literal)
+      (setq change (- (current-column) ocolumn))
+      (setq list1 buffer-undo-list)
+      (while list1
+	(setq list1 (primitive-undo 1 list1))))
+    (goto-char pos)
+    (if (> change 0)
+	(delete-region (point)
+		       (progn
+			 (move-to-column-force (+ change (current-column)))
+			 (point))))
+    (replace-match newtext fixedcase literal)
+    (if (< change 0)
+	(insert-char ?\ (- change)))))
 
 ;; Picture Tabs
 
@@ -317,7 +303,7 @@ It defines a set of \"interesting characters\" to look for when setting
 For example, suppose that you are editing a table which is formatted thus:
 | foo		| bar + baz | 23  *
 | bubbles	| and + etc | 97  *
-and that picture-tab-chars is \"|+*\".  Then invoking
+and that `picture-tab-chars' is \"|+*\".  Then invoking
 \\[picture-set-tab-stops] on either of the previous lines would result
 in the following tab stops
 		:     :     :     :
@@ -334,14 +320,13 @@ The command \\[picture-tab-search] is defined to move beneath (or to) a
 character belonging to this set independent of the tab stops list.")
 
 (defun picture-set-tab-stops (&optional arg)
-  "Set value of  tab-stop-list  according to context of this line.
-This controls the behavior of \\[picture-tab].  A tab stop
-is set at every column occupied by an \"interesting character\" that is
-preceded by whitespace.  Interesting characters are defined by the
-variable  picture-tab-chars,  see its documentation for an example
-of usage.  With ARG, just (re)set  tab-stop-list  to its default value.
-The tab stops computed are displayed in the minibuffer with `:' at
-each stop."
+  "Set value of `tab-stop-list' according to context of this line.
+This controls the behavior of \\[picture-tab].  A tab stop is set at
+every column occupied by an \"interesting character\" that is preceded
+by whitespace.  Interesting characters are defined by the variable
+`picture-tab-chars', see its documentation for an example of usage.
+With ARG, just (re)set `tab-stop-list' to its default value.  The tab
+stops computed are displayed in the minibuffer with `:' at each stop."
   (interactive "P")
   (save-excursion
     (let (tabs)
@@ -368,7 +353,7 @@ each stop."
   "Move to column beneath next interesting char in previous line.
 With ARG move to column occupied by next interesting character in this
 line.  The character must be preceded by whitespace.
-\"interesting characters\" are defined by variable  picture-tab-chars.
+\"interesting characters\" are defined by variable `picture-tab-chars'.
 If no such character is found, move to beginning of line."
   (interactive "P")
   (let ((target (current-column)))
@@ -392,20 +377,18 @@ If no such character is found, move to beginning of line."
       (beginning-of-line))))
 
 (defun picture-tab (&optional arg)
-  "Tab transparently (move) to next tab stop.
-With ARG overwrite the traversed text with spaces.  The tab stop
+  "Tab transparently (just move point) to next tab stop.
+With prefix arg, overwrite the traversed text with spaces.  The tab stop
 list can be changed by \\[picture-set-tab-stops] and \\[edit-tab-stops].
-See also documentation for variable  picture-tab-chars."
+See also documentation for variable `picture-tab-chars'."
   (interactive "P")
-  (let* ((opoint (point))
-	 (target (prog2 (tab-to-tab-stop)
-			(current-column)
-			(delete-region opoint (point)))))
-    (move-to-column-force target)
+  (let* ((opoint (point)))
+    (move-to-tab-stop)
     (if arg
-	(let (indent-tabs-mode)
+	(let (indent-tabs-mode
+	      (column (current-column)))
 	  (delete-region opoint (point))
-	  (indent-to target)))))
+	  (indent-to column)))))
 
 ;; Picture Rectangles
 
@@ -416,40 +399,27 @@ The contents can be retrieved by \\[picture-yank-rectangle]")
 (defun picture-clear-rectangle (start end &optional killp)
   "Clear and save rectangle delineated by point and mark.
 The rectangle is saved for yanking by \\[picture-yank-rectangle] and replaced
-with whitespace.  The previously saved rectangle, if any, is lost.
-With prefix argument, the rectangle is actually killed, shifting remaining
-text."
+with whitespace.  The previously saved rectangle, if any, is lost.  With
+prefix argument, the rectangle is actually killed, shifting remaining text."
   (interactive "r\nP")
   (setq picture-killed-rectangle (picture-snarf-rectangle start end killp)))
 
 (defun picture-clear-rectangle-to-register (start end register &optional killp)
   "Clear rectangle delineated by point and mark into REGISTER.
-The rectangle is saved in REGISTER and replaced with whitespace.
-With prefix argument, the rectangle is actually killed, shifting remaining
-text."
+The rectangle is saved in REGISTER and replaced with whitespace.  With
+prefix argument, the rectangle is actually killed, shifting remaining text."
   (interactive "r\ncRectangle to register: \nP")
   (set-register register (picture-snarf-rectangle start end killp)))
 
 (defun picture-snarf-rectangle (start end &optional killp)
   (let ((column (current-column))
-	(indent-tabs-mode nil)
-	markpos oldmark)
+	(indent-tabs-mode nil))
     (prog1 (save-excursion
-	     (save-excursion
-	       (goto-char (setq oldmark (mark)))
-	       (setq markpos (current-column)))
              (if killp
                  (delete-extract-rectangle start end)
                (prog1 (extract-rectangle start end)
-		 (clear-rectangle start end t))))
-      (move-to-column-force column)
-      ;; Make the mark point at the same column
-      ;; as it did before.
-      (set-marker (mark-marker) 
-		  (save-excursion
-		    (goto-char oldmark)
-		    (move-to-column-force markpos)
-		    (point))))))
+                      (clear-rectangle start end))))
+	   (move-to-column-force column))))
 
 (defun picture-yank-rectangle (&optional insertp)
   "Overlay rectangle saved by \\[picture-clear-rectangle]
@@ -494,28 +464,35 @@ Leaves the region surrounding the rectangle."
 
 (defconst picture-mode-map nil)
 
+(defun picture-substitute (oldfun newfun)
+  (substitute-key-definition oldfun newfun picture-mode-map global-map))
+
 (if (not picture-mode-map)
     (let ((i ?\ ))
       (setq picture-mode-map (make-keymap))
       (if (boundp 'EGG)			; 89.9.7 by K.Handa
 	  (while (< i ?\177)
-	    (aset picture-mode-map i 'egg-self-insert-command)
+	    (define-key picture-mode-map (make-string 1 i) 'egg-self-insert-command)
 	    (setq i (1+ i)))
 	(while (< i ?\177)
-	  (aset picture-mode-map i 'picture-self-insert)
+	  (define-key picture-mode-map (make-string 1 i) 'picture-self-insert)
 	  (setq i (1+ i))))
-      (define-key picture-mode-map "\C-f" 'picture-forward-column)
-      (define-key picture-mode-map "\C-b" 'picture-backward-column)
-      (define-key picture-mode-map "\C-d" 'picture-clear-column)
+      (picture-substitute 'forward-char 'picture-forward-column)
+      (picture-substitute 'backward-char 'picture-backward-column)
+      (picture-substitute 'delete-char 'picture-clear-column)
+      ;; There are two possibilities for what is normally on DEL.
+      (picture-substitute 'backward-delete-char-untabify 'picture-backward-clear-column)
+      (picture-substitute 'delete-backward-char 'picture-backward-clear-column)
+      (picture-substitute 'kill-line 'picture-clear-line)
+      (picture-substitute 'open-line 'picture-open-line)
+      (picture-substitute 'newline 'picture-newline)
+      (picture-substitute 'newline-andindent 'picture-duplicate-line)
+      (picture-substitute 'next-line 'picture-move-down)
+      (picture-substitute 'previous-line 'picture-move-up)
+      (picture-substitute 'beginning-of-line 'picture-beginning-of-line)
+      (picture-substitute 'end-of-line 'picture-end-of-line)
+
       (define-key picture-mode-map "\C-c\C-d" 'delete-char)
-      (define-key picture-mode-map "\177" 'picture-backward-clear-column)
-      (define-key picture-mode-map "\C-k" 'picture-clear-line)
-      (define-key picture-mode-map "\C-o" 'picture-open-line)
-      (define-key picture-mode-map "\C-m" 'picture-newline)
-      (define-key picture-mode-map "\C-j" 'picture-duplicate-line)
-      (define-key picture-mode-map "\C-n" 'picture-move-down)
-      (define-key picture-mode-map "\C-p" 'picture-move-up)
-      (define-key picture-mode-map "\C-e" 'picture-end-of-line)
       (define-key picture-mode-map "\e\t" 'picture-toggle-tab-state)
       (define-key picture-mode-map "\t" 'picture-tab)
       (define-key picture-mode-map "\e\t" 'picture-tab-search)
@@ -531,23 +508,22 @@ Leaves the region surrounding the rectangle."
       (define-key picture-mode-map "\C-c>" 'picture-movement-right)
       (define-key picture-mode-map "\C-c^" 'picture-movement-up)
       (define-key picture-mode-map "\C-c." 'picture-movement-down)
-      ;;; 89.9.7 by K.Handa
-      (define-key picture-mode-map "\C-ck" 'picture-insert-string)
-      (define-key picture-mode-map "\C-c~" 'picture-movement-wnw)
-      (define-key picture-mode-map "\C-c\"" 'picture-movement-ene)
-      (define-key picture-mode-map "\C-c?" 'picture-movement-wsw)
-      (define-key picture-mode-map "\C-c|" 'picture-movement-ese)
-      ;;; end of patch
       (define-key picture-mode-map "\C-c`" 'picture-movement-nw)
       (define-key picture-mode-map "\C-c'" 'picture-movement-ne)
       (define-key picture-mode-map "\C-c/" 'picture-movement-sw)
       (define-key picture-mode-map "\C-c\\" 'picture-movement-se)))
 
-(defvar edit-picture-hook nil
-  "If non-nil, it's value is called on entry to Picture mode.
-Picture mode is invoked by the command \\[edit-picture].")
+(defvar picture-mode-hook nil
+  "If non-nil, its value is called on entry to Picture mode.
+Picture mode is invoked by the command \\[picture-mode].")
 
-(defun edit-picture ()
+(defvar picture-mode-old-local-map)
+(defvar picture-mode-old-mode-name)
+(defvar picture-mode-old-major-mode)
+(defvar picture-mode-old-truncate-lines)
+
+;;;###autoload
+(defun picture-mode ()
   "Switch to Picture mode, in which a quarter-plane screen model is used.
 Printing characters replace instead of inserting themselves with motion
 afterwards settable by these commands:
@@ -563,11 +539,11 @@ The current direction is displayed in the mode line.  The initial
 direction is right.  Whitespace is inserted and tabs are changed to
 spaces when required by movement.  You can move around in the buffer
 with these commands:
-  C-p	  Move vertically to SAME column in previous line.
-  C-n	  Move vertically to SAME column in next line.
-  C-e	  Move to column following last non-whitespace character.
-  C-f	  Move right inserting spaces if required.
-  C-b	  Move left changing tabs to spaces if required.
+  \\[picture-move-down]	  Move vertically to SAME column in previous line.
+  \\[picture-move-up]	  Move vertically to SAME column in next line.
+  \\[picture-end-of-line]	  Move to column following last non-whitespace character.
+  \\[picture-forward-column]	  Move right inserting spaces if required.
+  \\[picture-backward-column]	  Move left changing tabs to spaces if required.
   C-c C-f Move in direction of current picture motion.
   C-c C-b Move in opposite direction of current picture motion.
   Return  Move to beginning of next line.
@@ -583,10 +559,10 @@ You can edit tabular text with these commands:
 You can manipulate text with these commands:
   C-d	  Clear (replace) ARG columns after point without moving.
   C-c C-d Delete char at point - the command normally assigned to C-d.
-  Delete  Clear (replace) ARG columns before point, moving back over them.
-  C-k	  Clear ARG lines, advancing over them.	 The cleared
+  \\[picture-backward-clear-column]  Clear (replace) ARG columns before point, moving back over them.
+  \\[picture-clear-line]	  Clear ARG lines, advancing over them.	 The cleared
 	    text is saved in the kill ring.
-  C-o	  Open blank line(s) beneath current line.
+  \\[picture-open-line]	  Open blank line(s) beneath current line.
 You can manipulate rectangles with these commands:
   C-c C-k Clear (or kill) a rectangle and save it.
   C-c C-w Like C-c C-k except rectangle is saved in named register.
@@ -599,13 +575,13 @@ You can return to the previous mode with:
   C-c C-c Which also strips trailing whitespace from every line.
 	    Stripping is suppressed by supplying an argument.
 
-Entry to this mode calls the value of  edit-picture-hook  if non-nil.
+Entry to this mode calls the value of  picture-mode-hook  if non-nil.
 
 Note that Picture mode commands will work outside of Picture mode, but
 they are not defaultly assigned to keys."
   (interactive)
-  (if (eq major-mode 'edit-picture)
-      (error "You are already editing a Picture.")
+  (if (eq major-mode 'picture-mode)
+      (error "You are already editing a picture.")
     (make-local-variable 'picture-mode-old-local-map)
     (setq picture-mode-old-local-map (current-local-map))
     (use-local-map picture-mode-map)
@@ -613,7 +589,7 @@ they are not defaultly assigned to keys."
     (setq picture-mode-old-mode-name mode-name)
     (make-local-variable 'picture-mode-old-major-mode)
     (setq picture-mode-old-major-mode major-mode)
-    (setq major-mode 'edit-picture)
+    (setq major-mode 'picture-mode)
     (make-local-variable 'picture-killed-rectangle)
     (setq picture-killed-rectangle nil)
     (make-local-variable 'tab-stop-list)
@@ -622,35 +598,34 @@ they are not defaultly assigned to keys."
     (setq picture-tab-chars (default-value 'picture-tab-chars))
     (make-local-variable 'picture-vertical-step)
     (make-local-variable 'picture-horizontal-step)
-    ;;; 89.9.17  patch by K.Handa
-    (make-local-variable 'picutre-old-self-insert-after-hook)
-    (setq picutre-old-self-insert-after-hook self-insert-after-hook)
-    (setq self-insert-after-hook 'picture-insert-after-hook)
-    ;;; end of patch
+    (make-local-variable 'picture-mode-old-truncate-lines)
+    (setq picture-mode-old-truncate-lines truncate-lines)
+    (setq truncate-lines t)
     (picture-set-motion 0 1)
+
+    ;; edit-picture-hook is what we used to run, picture-mode-hook is in doc.
     (run-hooks 'edit-picture-hook 'picture-mode-hook)
     (message
      (substitute-command-keys
       "Type \\[picture-mode-exit] in this buffer to return it to %s mode.")
      picture-mode-old-mode-name)))
 
-(fset 'picture-mode 'edit-picture)	; for the confused
+;;;###autoload
+(defalias 'edit-picture 'picture-mode)
 
 (defun picture-mode-exit (&optional nostrip)
-  "Undo edit-picture and return to previous major mode.
+  "Undo picture-mode and return to previous major mode.
 With no argument strips whitespace from end of every line in Picture buffer
   otherwise just return to previous mode."
   (interactive "P")
-  (if (not (eq major-mode 'edit-picture))
+  (if (not (eq major-mode 'picture-mode))
       (error "You aren't editing a Picture.")
     (if (not nostrip) (picture-clean))
     (setq mode-name picture-mode-old-mode-name)
     (use-local-map picture-mode-old-local-map)
     (setq major-mode picture-mode-old-major-mode)
-    ;;; 89.9.17  patch by K.Handa
-    (setq self-insert-after-hook picutre-old-self-insert-after-hook)
-    ;;; end of patch
     (kill-local-variable 'tab-stop-list)
+    (setq truncate-lines picture-mode-old-truncate-lines)
     ;; Kludge - force the mode line to be updated.  Is there a better
     ;; way to do this?
     (set-buffer-modified-p (buffer-modified-p))))
@@ -661,3 +636,7 @@ With no argument strips whitespace from end of every line in Picture buffer
    (goto-char (point-min))
    (while (re-search-forward "[ \t][ \t]*$" nil t)
      (delete-region (match-beginning 0) (point)))))
+
+(provide 'picture)
+
+;;; picture.el ends here
