@@ -28,11 +28,9 @@
 ;;;
 ;;; sj3-egg $B$K4X$9$kDs0F!"Cn>pJs$O(B kiyoji@sm.sony.co.jp $B$K$*Aw$j2<$5$$!#(B
 ;;;
-;;;                      $B")(B 141 $BEl5~ETIJ@n6hEl8^H?ED(B1-22-1 $B8^H?ED(B AN $B%S%k(B
-;;;                      $B%=%K!<3t<02q<R%Q!<%=%J%k%$%s%U%)%a!<%7%g%s;v6HK\It(B
-;;;
 ;;;                                                $B@P0f(B $B@6<!(B
 
+(require 'egg)
 (provide 'sj3-egg)
 (if (not (boundp 'SJ3))
     (require 'sj3-client))
@@ -69,8 +67,8 @@
 ;;;
 ;;;----------------------------------------------------------------------
 
-(defvar sj3-egg-version "2.04" "Version number of this version of Egg. ")
-;;; Last modified date: Wed Jul 21 17:54:20 1993
+(defvar sj3-egg-version "3.00" "Version number of this version of Egg. ")
+;;; Last modified date: Thu Aug  4 21:18:11 1994
 
 (and (equal (user-full-name) "Kiyoji Ishii")
      (defun sj3-egg-version-update (arg)
@@ -573,11 +571,29 @@ PATHNAME$B$O4D6-JQ?t$r4^$s$G$h$$!#(B"
 ;;;; User entry : henkan-region, henkan-paragraph, henkan-sentence
 ;;;;
 
-(defun egg:henkan-attribute-on ()
-  (egg:set-region-attribute egg:*henkan-attribute* t))
+(defconst egg:*bunsetu-face* nil "*$BJ8@aI=<($KMQ$$$k(B face $B$^$?$O(B nil")
+(make-variable-buffer-local
+ (defvar egg:*bunsetu-overlay* nil "$BJ8@a$NI=<($K;H$&(B overlay"))
 
-(defun egg:henkan-attribute-off ()
-  (egg:set-region-attribute egg:*henkan-attribute* nil))
+(defconst egg:*bunsetu-kugiri* " " "*$BJ8@a$N6h@Z$j$r<($9J8;zNs(B")
+
+
+(defconst egg:*henkan-face* nil "*$BJQ49NN0h$rI=<($9$k(B face $B$^$?$O(B nil")
+(make-variable-buffer-local
+ (defvar egg:*henkan-overlay* nil "$BJQ49NN0h$NI=<($K;H$&(B overlay"))
+
+(defconst egg:*henkan-open*  "|" "*$BJQ49$N;OE@$r<($9J8;zNs(B")
+(defconst egg:*henkan-close* "|" "*$BJQ49$N=*E@$r<($9J8;zNs(B")
+
+(defun egg:henkan-face-on ()
+  (if (overlayp egg:*henkan-overlay*) nil
+    (setq egg:*henkan-overlay* (make-overlay 1 1 nil))
+    (overlay-put egg:*henkan-overlay* 'face egg:*henkan-face*) )
+  (move-overlay egg:*henkan-overlay* egg:*region-start* egg:*region-end* (current-buffer)) )
+
+(defun egg:henkan-face-off ()
+  (and (overlayp egg:*henkan-overlay*)
+       (delete-overlay egg:*henkan-overlay*) ))
 
 (defun henkan-region (start end)
   (interactive "r")
@@ -610,11 +626,11 @@ PATHNAME$B$O4D6-JQ?t$r4^$s$G$h$$!#(B"
 		(set-marker egg:*region-start* (point))
 		(insert egg:*henkan-close*)
 		(set-marker egg:*region-end* egg:*region-start*)
-		(egg:henkan-attribute-on)
+		(egg:henkan-face-on)
 		(goto-char egg:*region-start*)
 		)
 	    (progn
-	      (egg:fence-attribute-off)
+	      (egg:fence-face-off)
 	      (delete-region (- egg:*region-start* (length egg:*fence-open*)) 
 			     egg:*region-start*)
 	      (delete-region egg:*region-end* (+ egg:*region-end* (length egg:*fence-close*)))
@@ -627,7 +643,7 @@ PATHNAME$B$O4D6-JQ?t$r4^$s$G$h$$!#(B"
 		(set-marker egg:*region-end* point))
 	      (goto-char start)
 	      (delete-region start end)
-	      (egg:henkan-attribute-on))
+	      (egg:henkan-face-on))
 	    )
 	  (henkan-insert-kouho 0)
 	  (henkan-goto-bunsetu 0)
@@ -668,43 +684,40 @@ PATHNAME$B$O4D6-JQ?t$r4^$s$G$h$$!#(B"
 ;;; Kana Kanji Henkan Henshuu mode
 ;;;
 
-(defconst egg:*bunsetu-kugiri* " " "*$BJ8@a$N6h@Z$j$r<($9J8;zNs(B")
-(defconst egg:*bunsetu-attribute* nil "*$BJ8@aI=<($KMQ$$$k(Battribute $B$^$?$O(B nil")
-
-(defconst egg:*henkan-attribute* nil "*$BJQ49NN0h$rI=<($9$k(Battribute $B$^$?$O(B nil")
-(defconst egg:*henkan-open*  "|" "*$BJQ49$N;OE@$r<($9J8;zNs(B")
-(defconst egg:*henkan-close* "|" "*$BJQ49$N=*E@$r<($9J8;zNs(B")
-
-(defun set-egg-henkan-mode-format (open close kugiri &optional attr1 attr2)
-   "$BJQ49(B mode $B$NI=<(J}K!$r@_Dj$9$k!#(BOPEN $B$OJQ49$N;OE@$r<($9J8;zNs$^$?$O(B nil$B!#(B\n\
-CLOSE$B$OJQ49$N=*E@$r<($9J8;zNs$^$?$O(B nil$B!#(B\n\
-KUGIRI$B$OJ8@a$N6h@Z$j$rI=<($9$kJ8;zNs$^$?$O(B nil$B!#(B\n\
-optional ATTR1 $B$OJQ496h4V$rI=<($9$kB0@-(B $B$^$?$O(B nil$B!J(Bx11term $B$N$_$GM-8z!K(B\n\
-optional ATTR2 $B$OJ8@a6h4V$rI=<($9$kB0@-(B $B$^$?$O(B nil$B!J(Bx11term $B$N$_$GM-8z!K(B"
+(defun set-egg-henkan-mode-format (open close kugiri &optional henkan-face bunsetu-face)
+   "$BJQ49(B mode $B$NI=<(J}K!$r@_Dj$9$k!#(BOPEN $B$OJQ49$N;OE@$r<($9J8;zNs$^$?$O(B nil$B!#(B
+CLOSE$B$OJQ49$N=*E@$r<($9J8;zNs$^$?$O(B nil$B!#(B
+KUGIRI$B$OJ8@a$N6h@Z$j$rI=<($9$kJ8;zNs$^$?$O(B nil$B!#(B
+HENKAN-FACE $B$,;XDj$5$l$F(B nil $B$G$J$1$l$P!"JQ496h4V$rI=<($9$k(B face $B$H$7$F;H$o$l$k!#(B
+BUNSETU-FACE $B$,;XDj$5$l$F(B nil $B$G$J$1$l$P!"CmL\$7$F$$$kJ8@a$rI=<($9$k(B face $B$H$7$F;H$o$l$k(B"
 
   (interactive (list (read-string "$BJQ493+;OJ8;zNs(B: ")
 		     (read-string "$BJQ49=*N;J8;zNs(B: ")
 		     (read-string "$BJ8@a6h@Z$jJ8;zNs(B: ")
-		     (cdr (assoc (completing-read "$BJQ496h4VI=<(B0@-(B: " egg:*attribute-alist*)
-				 egg:*attribute-alist*))
-		     (cdr (assoc (completing-read "$BJ8@a6h4VI=<(B0@-(B: " egg:*attribute-alist*)
-				 egg:*attribute-alist*))
+		     (cdr (assoc (completing-read "$BJQ496h4VI=<(B0@-(B: " egg:*face-alist*)
+				 egg:*face-alist*))
+		     (cdr (assoc (completing-read "$BJ8@a6h4VI=<(B0@-(B: " egg:*face-alist*)
+				 egg:*face-alist*))
 		     ))
 
   (if (and (or (stringp open)  (null open))
 	   (or (stringp close) (null close))
 	   (or (stringp kugiri) (null kugiri))
-	   (egg:member attr1 '(underline inverse bold nil))
-	   (egg:member attr2 '(underline inverse bold nil)))
+	   (or (null henkan-face) (memq henkan-face (face-list)))
+	   (or (null bunsetu-face) (memq henkan-face (face-list))))
       (progn
-	(setq egg:*henkan-open* open
-	      egg:*henkan-close* close
+	(setq egg:*henkan-open* (or open "")
+	      egg:*henkan-close* (or close "")
 	      egg:*bunsetu-kugiri* (or kugiri "")
-	      egg:*henkan-attribute* attr1
-	      egg:*bunsetu-attribute* attr2)
-	(if (or attr1 attr2) (require 'attribute))
-	nil)
-    (error "Wrong type of arguments: %1 %2 %3 %4 %5" open close kugiri attr1 attr2)))
+	      egg:*henkan-face* henkan-face
+	      egg:*bunsetu-face* bunsetu-face)
+	(if (overlayp egg:*henkan-overlay*)
+	    (overlay-put egg:*henkan-overlay* 'face egg:*henkan-face*))
+	(if (overlayp egg:*bunsetu-overlay*)
+	    (overlay-put egg:*bunsetu-overlay* 'face egg:*bunsetu-face*))
+
+	t)
+    (error "Wrong type of arguments: %1 %2 %3 %4 %5" open close kugiri henkan-face bunsetu-face)))
 
 (defun henkan-insert-kouho (bunsetu-no)
   (let ((max (bunsetu-su)) (i bunsetu-no))
@@ -716,8 +729,8 @@ optional ATTR2 $B$OJ8@a6h4V$rI=<($9$kB0@-(B $B$^$?$O(B nil$B!J(Bx11term $
 
 (defun henkan-kakutei ()
   (interactive)
-  (egg:bunsetu-attribute-off *bunsetu-number*)
-  (egg:henkan-attribute-off)
+  (egg:bunsetu-face-off *bunsetu-number*)
+  (egg:henkan-face-off)
   (delete-region (- egg:*region-start* (length egg:*henkan-open*))
 		 egg:*region-start*)
   (delete-region egg:*region-start* egg:*region-end*)
@@ -737,8 +750,8 @@ optional ATTR2 $B$OJ8@a6h4V$rI=<($9$kB0@-(B $B$^$?$O(B nil$B!J(Bx11term $
 
 (defun henkan-kakutei-before-point ()
   (interactive)
-  (egg:bunsetu-attribute-off *bunsetu-number*)
-  (egg:henkan-attribute-off)
+  (egg:bunsetu-face-off *bunsetu-number*)
+  (egg:henkan-face-off)
   (delete-region egg:*region-start* egg:*region-end*)
   (goto-char egg:*region-start*)
   (let ((i 0) (max *bunsetu-number*))
@@ -760,7 +773,7 @@ optional ATTR2 $B$OJ8@a6h4V$rI=<($9$kB0@-(B $B$^$?$O(B nil$B!J(Bx11term $
     (insert egg:*fence-close*)
     (set-marker egg:*region-end* point))
   (goto-char egg:*region-start*)
-  (egg:fence-attribute-on)
+  (egg:fence-face-on)
   (let ((point (point))
 	(i *bunsetu-number*) (max (bunsetu-su)))
     (while (< i max)
@@ -776,39 +789,37 @@ optional ATTR2 $B$OJ8@a6h4V$rI=<($9$kB0@-(B $B$^$?$O(B nil$B!J(Bx11term $
   (use-local-map fence-mode-map)
   (egg:mode-line-display))
 
-(defun egg:set-region-attribute (attr on)
-  (if attr 
-      (attribute-on-off-region attr egg:*region-start* egg:*region-end* on)))
+(defun egg:set-bunsetu-face (no face switch)
+  (if (not switch)
+      (egg:bunsetu-face-off no) ;; JIC
+    (if (overlayp egg:*bunsetu-overlay*) nil
+      (setq egg:*bunsetu-overlay* (make-overlay 1 1 nil))
+      (overlay-put egg:*bunsetu-overlay* 'face egg:*bunsetu-face*))
+    (move-overlay egg:*bunsetu-overlay*
+		  (if (eq face 'modeline)
+		      (let ((point (bunsetu-position no)))
+			(+ point (1+ (char-boundary-p point))))
+		    (bunsetu-position no))
 
-(defun egg:set-bunsetu-attribute (no attr switch)
-  (if (and no attr)
-      (attribute-on-off-region
-       attr
-       (if (eq attr 'inverse)
-	   (let ((point (bunsetu-position no)))
-	     (+ point (1+ (char-boundary-p point))))
-	 (bunsetu-position no))
+		  (if (= no (1- (bunsetu-su)))
+		      egg:*region-end*
+		    (- (bunsetu-position (1+ no))
+		       (length egg:*bunsetu-kugiri*)))
+		  (current-buffer))))
 
-       (if (= no (1- (bunsetu-su)))
-	   egg:*region-end*
-	 (- (bunsetu-position (1+ no))
-	    (length egg:*bunsetu-kugiri*)))
-       switch)))
+(defun egg:bunsetu-face-on (no)
+  (egg:set-bunsetu-face no egg:*bunsetu-face* t))
 
-(defun egg:bunsetu-attribute-on (no)
-  (egg:set-bunsetu-attribute no egg:*bunsetu-attribute* t))
-
-(defun egg:bunsetu-attribute-off (no)
-  (egg:set-bunsetu-attribute no egg:*bunsetu-attribute* nil))
+(defun egg:bunsetu-face-off (no)
+  ;; ``no'' will be ignored
+  (and (overlayp egg:*bunsetu-overlay*)
+       (delete-overlay egg:*bunsetu-overlay*)) )
 
 (defun henkan-goto-bunsetu (number)
-  (egg:bunsetu-attribute-off *bunsetu-number*)
-  (egg:henkan-attribute-on)
   (setq *bunsetu-number*
 	(check-number-range number 0 (1- (bunsetu-su))))
   (goto-char (bunsetu-position *bunsetu-number*))
-  (egg:set-bunsetu-attribute *bunsetu-number* egg:*henkan-attribute* nil)
-  (egg:bunsetu-attribute-on *bunsetu-number*)
+  (egg:bunsetu-face-on *bunsetu-number*)
   )
 
 (defun henkan-forward-bunsetu ()
@@ -852,8 +863,6 @@ optional ATTR2 $B$OJ8@a6h4V$rI=<($9$kB0@-(B $B$^$?$O(B nil$B!J(Bx11term $
   (henkan-goto-kouho (1- (bunsetu-kouho-number *bunsetu-number*))))
 
 (defun henkan-goto-kouho (kouho-number)
-  (egg:bunsetu-attribute-off *bunsetu-number*)
-  (egg:henkan-attribute-on)
   (let ((point (point))
 	(yomi  (bunsetu-yomi *bunsetu-number*))
 	(i *bunsetu-number*)
@@ -872,8 +881,7 @@ optional ATTR2 $B$OJ8@a6h4V$rI=<($9$kB0@-(B $B$^$?$O(B nil$B!J(Bx11term $
 	    (bunsetu-kanji-insert i)))
       (setq i (1+ i)))
     (goto-char point))
-  (egg:set-bunsetu-attribute *bunsetu-number* egg:*henkan-attribute* nil)
-  (egg:bunsetu-attribute-on *bunsetu-number*))
+  (egg:bunsetu-face-on *bunsetu-number*))
 
 (defun henkan-bunsetu-chijime ()
   (interactive)
@@ -899,8 +907,6 @@ optional ATTR2 $B$OJ8@a6h4V$rI=<($9$kB0@-(B $B$^$?$O(B nil$B!J(Bx11term $
     (bunsetu-length-henko l)))
 
 (defun bunsetu-length-henko (length)
-  (egg:henkan-attribute-off)
-  (egg:bunsetu-attribute-off *bunsetu-number*)
   (let ((r (KKCP:bunsetu-henkou *bunsetu-number* length)))
     (cond(r
 	  (delete-region 
@@ -909,14 +915,12 @@ optional ATTR2 $B$OJ8@a6h4V$rI=<($9$kB0@-(B $B$^$?$O(B nil$B!J(Bx11term $
 	  (henkan-insert-kouho *bunsetu-number*)
 	  (henkan-goto-bunsetu *bunsetu-number*))
 	 (t
-	  (egg:henkan-attribute-on)
-	  (egg:set-bunsetu-attribute *bunsetu-number* egg:*henkan-attribute* nil)
-	  (egg:bunsetu-attribute-on *bunsetu-number*)))))
+	  (egg:bunsetu-face-on *bunsetu-number*)))))
 
 (defun henkan-quit ()
   (interactive)
-  (egg:bunsetu-attribute-off *bunsetu-number*)
-  (egg:henkan-attribute-off)
+  (egg:bunsetu-face-off *bunsetu-number*)
+  (egg:henkan-face-off)
   (delete-region (- egg:*region-start* (length egg:*henkan-open*))
 		 egg:*region-start*)
   (delete-region egg:*region-start* egg:*region-end*)
@@ -930,7 +934,7 @@ optional ATTR2 $B$OJ8@a6h4V$rI=<($9$kB0@-(B $B$^$?$O(B nil$B!J(Bx11term $
     (set-marker egg:*region-end* point)
     )
   (goto-char egg:*region-end*)
-  (egg:fence-attribute-on)
+  (egg:fence-face-on)
   (KKCP:henkan-quit)
   (setq egg:*mode-on* t)
   ;;;(use-global-map fence-mode-map)
@@ -959,7 +963,7 @@ optional ATTR2 $B$OJ8@a6h4V$rI=<($9$kB0@-(B $B$^$?$O(B nil$B!J(Bx11term $
 
 (defun henkan-kakutei-and-self-insert ()
   (interactive)
-  (setq unread-command-char last-command-char)
+  (setq unread-command-events (list last-command-char))
   (henkan-kakutei))
 
 
@@ -969,13 +973,13 @@ optional ATTR2 $B$OJ8@a6h4V$rI=<($9$kB0@-(B $B$^$?$O(B nil$B!J(Bx11term $
 
 (let ((ch 0))
   (while (<= ch 127)
-    (aset henkan-mode-map ch 'undefined)
-    (aset henkan-mode-esc-map ch 'undefined)
+    (define-key henkan-mode-map (make-string 1 ch) 'undefined)
+    (define-key henkan-mode-esc-map (make-string 1 ch) 'undefined)
     (setq ch (1+ ch))))
 
 (let ((ch 32))
   (while (< ch 127)
-    (aset henkan-mode-map ch 'henkan-kakutei-and-self-insert)
+    (define-key henkan-mode-map (make-string 1 ch) 'henkan-kakutei-and-self-insert)
     (setq ch (1+ ch))))
 	
 (define-key henkan-mode-map "\e"    henkan-mode-esc-map)
@@ -1092,7 +1096,19 @@ optional ATTR2 $B$OJ8@a6h4V$rI=<($9$kB0@-(B $B$^$?$O(B nil$B!J(Bx11term $
 ;;; (lsh 1 18)
 (defvar *sj3-bunpo-menu*
   '(menu "$BIJ;l(B:"
-   (("$BL>;l(B"      . 1)
+   (("$BL>;l(B"      .
+     (menu "$BIJ;l(B:$BL>;l(B:"
+	   (("$BL>;l(B"		. 1)
+	    ("$BL>;l(B($B$*!D(B)"	. 2)
+	    ("$BL>;l(B($B$4!D(B)"	. 3)
+	    ("$BL>;l(B($B!DE*(B/$B2=(B)"	. 4)
+	    ("$BL>;l(B($B!D$9$k(B)"	. 6)
+	    ("$BL>;l(B($B$*!D$9$k(B)"	. 5)
+	    ("$BL>;l(B($B$4!D$9$k(B)"	. 7)
+	    ("$BL>;l(B($B!D$J(B/$B$K(B)"	. 8)
+	    ("$BL>;l(B($B$*!D$J(B/$B$K(B)"	. 9)
+	    ("$BL>;l(B($B$4!D$J(B/$B$K(B)"	. 10)
+	    ("$BL>;l(B($BI{;l(B)"	. 11))))
     ("$BBeL>;l(B"    . 12)
     ("$BID;z(B"      . 21)
     ("$BL>A0(B"      . 22)
@@ -1119,6 +1135,7 @@ optional ATTR2 $B$OJ8@a6h4V$rI=<($9$kB0@-(B $B$^$?$O(B nil$B!J(Bx11term $
     ("$B@\F,8l(B"         . 31)
     ("$B@\Hx8l(B"         . 36)
     ("$BI{;l(B"           . 45)
+    ("$BI{;l(B2"          . 46)
     ("$B7AMF;l8l44(B"     . 60)
     ("$B7AMFF0;l8l44(B"   . 71)
     ("$BC14A;z(B"         . 189))))
@@ -1126,6 +1143,16 @@ optional ATTR2 $B$OJ8@a6h4V$rI=<($9$kB0@-(B $B$^$?$O(B nil$B!J(Bx11term $
 (defvar *sj3-bunpo-code*
   '(
     ( 1   "$BL>;l(B" )
+    ( 2   "$BL>;l(B($B$*!D(B)" )
+    ( 3   "$BL>;l(B($B$4!D(B)" )
+    ( 4   "$BL>;l(B($B!DE*(B/$B2=(B)" "$BE*(B" nil)
+    ( 5   "$BL>;l(B($B!D$9$k(B)" "$B$9$k(B" nil)
+    ( 6   "$BL>;l(B($B$*!D$9$k(B)" "$B$9$k(B" nil)
+    ( 7   "$BL>;l(B($B$4!D$9$k(B)" "$B$9$k(B" nil)
+    ( 8   "$BL>;l(B($B!D$J(B/$B$K(B)" "$B$J(B/$B$K(B" nil)
+    ( 9   "$BL>;l(B($B$*!D$J(B/$B$K(B)" "$B$J(B/$B$K(B" nil)
+    ( 10  "$BL>;l(B($B$4!D$J(B/$B$K(B)" "$B$J(B/$B$K(B" nil)
+    ( 11  "$BL>;l(B($BI{;l(B)" )
     ( 12  "$BBeL>;l(B" )
     ( 21  "$BID;z(B" )
     ( 22  "$BL>A0(B" )
@@ -1138,6 +1165,7 @@ optional ATTR2 $B$OJ8@a6h4V$rI=<($9$kB0@-(B $B$^$?$O(B nil$B!J(Bx11term $
     ( 31  "$B@\F,8l(B" )
     ( 36  "$B@\Hx8l(B" )
     ( 45  "$BI{;l(B" )
+    ( 46  "$BI{;l(B2" )
     ( 60  "$B7AMF;l8l44(B"           "$B$$(B" ("" "" "" "" ""))
     ( 71  "$B7AMFF0;l8l44(B"         "$B$K(B" ("" "" "" "" "") )
     ( 80  "$B%5JQ8l44(B"             "$B$9$k(B" ("" "" "" "" ""))
