@@ -1,21 +1,21 @@
 /*
-  CANNA interface for Mule
+  CANNA interface for NEmacs
 
-  This file is a part of Canna on Mule.
+  This file is a part of Canna on Nemacs.
 
-  Canna on Mule is distributed in the forms of patches
-  to Mule under the terms of the GNU EMACS GENERAL
+  Canna on Nemacs is distributed in the forms of patches
+  to Nemacs under the terms of the GNU EMACS GENERAL
   PUBLIC LICENSE which is distributed along with GNU Emacs
   by the Free Software Foundation.
 
-  Canna on Mule is distributed in the hope that it will
+  Canna on Nemacs is distributed in the hope that it will
   be useful, but WITHOUT ANY WARRANTY; without even the
   implied warranty of MERCHANTABILITY or FITNESS FOR A
   PARTICULAR PURPOSE.  See the GNU EMACS GENERAL PUBLIC
   LICENSE for more details.
 
   You should have received a copy of the GNU EMACS GENERAL
-  PUBLIC LICENSE along with Mule; see the file COPYING.
+  PUBLIC LICENSE along with Nemacs; see the file COPYING.
   If not, write to the Free Software Foundation, 675 Mass
   Ave, Cambridge, MA 02139, USA.
 
@@ -141,17 +141,16 @@
  */
 
 #ifndef lint
-static char rcs_id[] = "$Id: canna.c,v 1.32 1993/07/05 01:04:58 kon Exp $";
+static char rcs_id[] = "$Id: canna.c,v 1.35 1994/03/15 08:00:03 kon Exp $";
 #endif
 
 #include <stdio.h>
 
 #include "config.h"
-#undef NULL
 #include "lisp.h"
 #include "buffer.h"
 #ifdef CANNA_MULE
-#include "mule.h"
+#include "charset.h"
 #endif
 
 #ifdef CANNA2
@@ -170,6 +169,7 @@ static char **warning;
 
 static int Vcanna_empty_info, Vcanna_through_info;
 static int Vcanna_underline;
+static int Vcanna_inhibit_hankakukana;		/* hir, 1994.12.5 */
 
 static Lisp_Object Vcanna_kakutei_string;
 static Lisp_Object Vcanna_kakutei_yomi;
@@ -197,12 +197,12 @@ static Lisp_Object kanjiYomiList();
 #ifdef CANNA_MULE
 #if __STDC__
 static m2c(unsigned char *, int, unsigned char *);
-static mule_make_string(unsigned char *, int);
+static Lisp_Object mule_make_string(unsigned char *, int);
 static mule_strlen(unsigned char *, int);
 static count_char(unsigned char *,int, int, int, int *, int *, int *);
 #else
 static m2c();
-static mule_make_string();
+static Lisp_Object mule_make_string();
 static mule_strlen();
 static count_char();
 #endif
@@ -314,6 +314,21 @@ jrKanjiStatus *ks;
   return val;
 }
 
+DEFUN ("canna-set-bunsetsu-kugiri", Fcanna_set_bunsetsu, Scanna_set_bunsetsu,
+       0, 1, 0,
+"This function sets the clause separator.\n\
+If non-nil value is specified, the white space separator will be used.\n\
+No separator will be used otherwise.")
+(num)
+Lisp_Object num;
+{
+  int kugiri; /* ʸڤ򤹤뤫 */
+
+  kugiri = NILP(num) ? 0 : 1;
+
+  jrKanjiControl(0, KC_SETBUNSETSUKUGIRI, (char *)kugiri);
+}
+
 DEFUN ("canna-initialize", Fcanna_initialize, Scanna_initialize, 0, 3, 0,
 "Initialize ``canna'', which is a kana-to-kanji converter for GNU Emacs.\n\
 The first arg specifies if inserting space character between BUNSETSU when\n\
@@ -323,7 +338,7 @@ The third arg specifies a file which will be used as a customization\n\
 description.\n\
 If nil is specified for each arg, the default value will be used.")
 (num, server, rcfile)
-Lisp_Object num, server;
+Lisp_Object num, server, rcfile;
 {
   Lisp_Object val;
   int res;
@@ -333,7 +348,7 @@ Lisp_Object num, server;
 
   IRCP_context = -1;
 
-  if (NULL(num)) {
+  if (NILP(num)) {
     kugiri = 1;
   }
   else {
@@ -342,7 +357,7 @@ Lisp_Object num, server;
     kugiri = (kugiri == 1) ? 1 : 0;
   }
 
-  if (NULL(server)) {
+  if (NILP(server)) {
     jrKanjiControl(0, KC_SETSERVERNAME, (char *)0);
   }
   else {
@@ -354,7 +369,7 @@ Lisp_Object num, server;
     jrKanjiControl(0, KC_SETSERVERNAME, servername);
   }
 
-  if (NULL(rcfile)) {
+  if (NILP(rcfile)) {
     jrKanjiControl(0, KC_SETINITFILENAME, (char *)0);
   }
   else {
@@ -367,7 +382,13 @@ Lisp_Object num, server;
   }
 
   warning = (char **)0;
+#ifdef nec_ews_svr4				/* hir, 1994.2.24 */
+  stop_polling ();
+#endif /* nec_ews_svr4 */
   res = jrKanjiControl(0, KC_INITIALIZE, (char *)&warning);
+#ifdef nec_ews_svr4				/* hir, 1994.2.24 */
+  start_polling ();
+#endif /* nec_ews_svr4 */
   val = Qnil;
   if (warning) {
     for (p = q = (unsigned char **)warning ; *q ; q++)
@@ -402,8 +423,11 @@ Lisp_Object num, server;
     jrKanjiControl(0, KC_SETBUNSETSUKUGIRI, (char *)kugiri);
     jrKanjiControl(0, KC_SETWIDTH, (char *)78);
 #ifndef CANNA_MULE
-    /* mule äȾѥʤȤ */
     jrKanjiControl(0, KC_INHIBITHANKAKUKANA, (char *)1);
+#else					/* hir, 1994.12.5 */
+    /* mule äȾѥʤȤ */
+    if (Vcanna_inhibit_hankakukana)
+      jrKanjiControl(0, KC_INHIBITHANKAKUKANA, (char *)1);
 #endif
     jrKanjiControl(0, KC_YOMIINFO, (char *)2); /* : ޻ޤ֤ */
     val = Fcons(Qnil, val);
@@ -533,7 +557,7 @@ Lisp_Object yomi, roma;
   ks.length = strlen(buf);
 #endif /* CANNA_MULE */
 
-  if (NULL(roma)) {
+  if (NILP(roma)) {
     ks.mode = 0;
   }
   else {
@@ -572,7 +596,7 @@ Lisp_Object num, ch;
 
   CHECK_NUMBER(num, 0);
 
-  if (NULL(ch)) {
+  if (NILP(ch)) {
     *buf = '@';
   }
   else {
@@ -973,6 +997,7 @@ syms_of_canna ()
   defsubr (&Scanna_do_function);
   defsubr (&Scanna_parse);
   defsubr (&Scanna_query_mode);
+  defsubr (&Scanna_set_bunsetsu);
 
   DEFVAR_LISP("canna-kakutei-string", &Vcanna_kakutei_string, "");
   DEFVAR_LISP("canna-kakutei-yomi",   &Vcanna_kakutei_yomi,   "");
@@ -990,6 +1015,8 @@ syms_of_canna ()
   DEFVAR_BOOL ("canna-empty-info", &Vcanna_empty_info, "For canna");
   DEFVAR_BOOL ("canna-through-info", &Vcanna_through_info, "For canna");
   DEFVAR_BOOL ("canna-underline", &Vcanna_underline, "For canna");
+  DEFVAR_BOOL ("canna-inhibit-hankakukana", &Vcanna_inhibit_hankakukana, 
+	       "For canna");			/* hir, 1994.12.5 */
 
   defsubr (&Scanna_henkan_begin);
   defsubr (&Scanna_henkan_next);
@@ -1178,7 +1205,7 @@ syms_of_canna ()
 /* EUC multibyte string to MULE internal string */
 
 static
-c2m(cp, l, mp)
+c2mu(cp, l, mp)
 char	*cp;
 int	l;
 char	*mp;
@@ -1238,13 +1265,14 @@ unsigned char	*cp;
 
 /* make_string after converting EUC string to MULE internal string */
 static
+Lisp_Object
 mule_make_string(p,l)
 unsigned char *p;
 int l;
 {
   unsigned char cbuf[4096];
   
-  c2m(p,l,cbuf);
+  c2mu(p,l,cbuf);
   return (make_string(cbuf,strlen(cbuf)));
 }	
 
