@@ -31,6 +31,8 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */
 /* 93.7.12  modified for Mule Ver.0.9.8 by K.Handa <handa@etl.go.jp>
 	category_table_version introduced.
 	New function check_category(). */
+/* 94.2.23  modified for Mule Ver.1.1 by K.Handa <handa@etl.go.jp>
+	describe-category displays multilingual text. */
 
 #include "config.h"
 #include <ctype.h>
@@ -39,7 +41,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */
 #include "buffer.h"
 #include "syntax.h"
 #include "category.h"
-#include "mule.h"
+#include "charset.h"
 
 unsigned int category_table_version; /* 93.7.12 by K.Handa */
 
@@ -64,9 +66,9 @@ check_category_table (obj)
      Lisp_Object obj;
 {
   register Lisp_Object tem;
-  if (NULL (obj)) return current_buffer->category_table;
+  if (NILP (obj)) return current_buffer->category_table;
   tem = Fcategory_table_p (obj);
-  if (NULL (tem))
+  if (NILP (tem))
     obj = wrong_type_argument (Qcategory_table_p, obj, 0);
   return obj;
 }   
@@ -168,7 +170,7 @@ You can't 'new-category-mnemonic' with the returned mnemonic.")
 
   temp = XVECTOR (ctbl)->contents[256];
   for (i = 0; i < 95; i++)
-    if (!NULL (XVECTOR (temp)->contents[i])) *p++ = i + ' ';
+    if (!NILP (XVECTOR (temp)->contents[i])) *p++ = i + ' ';
   return (p == str ? Qnil : make_string (str, p - str));
 }
 
@@ -190,7 +192,7 @@ before really using it.")
 
   temp = XVECTOR (ctbl)->contents[256];
   for (i = 0; i < 95; i++)
-    if (NULL (XVECTOR (temp)->contents[i])) *p++ = i + ' ';
+    if (NILP (XVECTOR (temp)->contents[i])) *p++ = i + ' ';
   return (p == str ? Qnil : make_string (str, p - str));
 }
 
@@ -245,9 +247,9 @@ It is a copy of the TABLE, which defaults to the standard category table.")
   (table)
      Lisp_Object table;
 {
-  if (!NULL (table))
+  if (!NILP (table))
     table = check_category_table (table);
-  else if (NULL (Vstandard_category_table))
+  else if (NILP (Vstandard_category_table))
     /* Can only be null during initialization */
     return make_init_category_table(257);
   else table = Vstandard_category_table;
@@ -448,61 +450,73 @@ On success, returns T, else returns NIL.")
   ctbl = check_category_table (ctbl);
 
   m = XFASTINT (mnemonic) - ' ';
-  if (NULL (XVECTOR (XVECTOR (ctbl)->contents[256])->contents[m]))
+  if (NILP (XVECTOR (XVECTOR (ctbl)->contents[256])->contents[m]))
     error ("Invalid mnemonic: %c", m + ' ');
   
-  modify_category_entry (XFASTINT (c), m, ctbl, !NULL(reset));
+  modify_category_entry (XFASTINT (c), m, ctbl, !NILP(reset));
   category_table_version++;	/* 93.7.12 by K.Handa */
   return Qnil;
 }
 
 /* Dump category table to buffer in human-readable format */
 
-insert_character_description(i)
+insert_character_description(i)	/* 94.2.23 by K.Handa */
      unsigned int i;
 {				/* 93.6.7 by K.Handa */
-  char str[6];			/* 92.7.10 by T.Enami */
+  unsigned char str[5];		/* 92.7.10 by T.Enami */
 
   if (i <= ' '){
-    char *p = (char *)push_key_description (i, str);
+    unsigned char *p = (unsigned char *)push_key_description (i, str);
     insert (str, p - str);
   } else if (i <= '~') {
     str[0] = i;
     insert (str, 1);
   } else {
-    if (NULL (current_buffer->ctl_hexa)) /* 93.6.7 by K.Handa */
-      sprintf(str, "\\%03o", i);
-    else
-      sprintf(str, "\\x%02x", i);
-    insert (str, 4);
+    int j = i & 0xFF000000 ? 3 : i & 0xFF0000 ? 2 : i & 0xFF00 ? 1 : 0;
+    int k = j;
+
+    while (i) str[j--] = i & 0xFF, i >>= 8;
+
+    if (char_bytes[str[0]] == k + 1) {
+      insert (str, k + 1);
+      sprintf(str, "[%02x]", str[k]);
+      insert (str, 4);
+    } else {
+      if (NILP (current_buffer->ctl_hexa)) /* 93.6.7 by K.Handa */
+	sprintf(str, "\\%03o", str[k]);
+      else
+	sprintf(str, "\\x%02x", str[k]);
+      insert (str, 4);
+    }
   }
 }
 
-describe_category (ctbl, indent)
+describe_category (ctbl, parent) /* 94.2.23 by K.Handae */
      Lisp_Object ctbl;
-     int indent;
+     int parent;
 {
   int i, start, deeper;
   char mnemonic, str[95];
   struct Lisp_Vector *v = XVECTOR (ctbl);
   Lisp_Object category;
+  int indent = !parent ? 0 : parent < 0x100 ? 2 : 4;
 
-  for (i = 0; i < 256; i++) {
+  for (i = parent ? 0xa0 : 0; i < 256; i++) {
     Findent_to (make_number (indent), make_number (0));
-    insert_character_description(i);
+    insert_character_description((parent << 8) | i);
     start = i;
     category = v->contents[i];
 
-    while (i + 1 < 256
-	   && !NULL (Fcategory_equal (v->contents[i + 1], category)))
+    while (i < 0xFF
+	   && !NILP (Fcategory_equal (v->contents[i + 1], category)))
       i++;
 
     if (i != start) {
       insert (" .. ", 4);
-      insert_character_description(i);
+      insert_character_description((parent << 8) | i);
     }
 
-    Findent_to (make_number (indent + 20), make_number (1));
+    Findent_to (make_number (24), make_number (1));
     
     deeper = 0;
     if (XTYPE (category) == Lisp_Cons) {
@@ -511,18 +525,18 @@ describe_category (ctbl, indent)
     }
 
     if (XTYPE (category) != Lisp_String || XSTRING (category)->size != 12) {
-      InsStr ("invalid");
+      insert_string ("invalid");
     } else {
       pack_mnemonic_string (XCATEGORY (category), str);
       if (strlen(str))
 	insert1 (build_string(str));
       else
-	InsStr ("No category");
+	insert_string ("No category");
     }
-    InsStr ("\n");
+    insert_string ("\n");
     if (deeper) {
       ctbl = XCONS (v->contents[i])->cdr;
-      describe_category (ctbl, indent + 2);
+      describe_category (ctbl, (parent << 8) | i);
     }
   }
 }
@@ -534,15 +548,15 @@ describe_mnemonic(description)
   struct Lisp_Vector *v = XVECTOR(description);
   char str[3];
 
-  InsStr ("\nMeanings of mnemonics:\n");
+  insert_string ("\nMeanings of mnemonics:\n");
   
   str[1] = ':'; str[2] = ' ';
   for (i = 0; i < 95; i++)
-    if (!NULL (v->contents[i])) {
+    if (!NILP (v->contents[i])) {
       str[0] = i + ' ';
       insert (str, 3);
       insert1 (v->contents[i]);
-      InsStr ("\n");
+      insert_string ("\n");
     }
 }
 
