@@ -1,11 +1,11 @@
 /* Fully extensible Emacs, running on Unix, intended for GNU.
-   Copyright (C) 1985, 1986, 1987, 1990 Free Software Foundation, Inc.
+   Copyright (C) 1985, 1986, 1987, 1993, 1994 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
 GNU Emacs is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 1, or (at your option)
+the Free Software Foundation; either version 2, or (at your option)
 any later version.
 
 GNU Emacs is distributed in the hope that it will be useful,
@@ -50,57 +50,24 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 	Support MCPATH. */
 /* 93.5.14  modified for Mule Ver.0.9.8 by K.Handa <handa@etl.go.jp>
 	In main(), call syms_of_ccl(). */
+/* 93.9.8   modified for Mule Ver.1.1 by A.Kon <kon@d1.bs2.mt.nec.co.jp>
+	Wnn and Canna can coexist. */
+/* 93.9.25  modified for Mule Ver.1.1 by T.Shingu <shingu@cpr.canon.co.jp>
+	wnn_exit() is obsolete. */
 
-/* This must precede sys/signal.h on certain machines.  */
-#include <sys/types.h>
 #include <signal.h>
 #include <errno.h>
 
-#include "config.h"
-#ifdef NULL
-#undef NULL
-#endif
-#include "lisp.h"
-#undef NULL
-#include "commands.h"
-
-/* Get FIONREAD, if it is available,
-   just to help decide whether SIGIO should be defined.  */
-#ifdef USG
-#include <termio.h>
-#include <fcntl.h>
-#else /* not USG */
-#ifndef VMS
-#include <sys/ioctl.h>
-#endif /* not VMS */
-#endif /* not USG */
-
-/* Allow m- file to inhibit use of FIONREAD.  */
-#ifdef BROKEN_FIONREAD
-#undef FIONREAD
-#endif
-
-/* We are unable to use interrupts if FIONREAD is not available,
-   so flush SIGIO so we won't try. */
-#ifndef FIONREAD
-#ifdef SIGIO
-#undef SIGIO
-#endif
-#endif
-
+#include <config.h>
 #include <stdio.h>
+
+#include <sys/types.h>
 #include <sys/file.h>
 
 #ifdef VMS
 #include <ssdef.h>
 #endif
 
-#if 0 /* fcntl.h was included above.  */
-#ifdef USG5
-#include <fcntl.h>
-#endif
-#endif
-
 #ifdef BSD
 #include <sys/ioctl.h>
 #endif
@@ -129,15 +96,36 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #endif
 /* end of patch */
 
+#include "lisp.h"
+#include "commands.h"
+#include "intervals.h"
+
+#include "systty.h"
+#include "syssignal.h"
+#include "process.h"
+
 #ifndef O_RDWR
 #define O_RDWR 2
 #endif
 
-#define PRIO_PROCESS 0
+extern void malloc_warning ();
+extern char *index ();
+extern char *strerror ();
 
 /* Command line args from shell, as list of strings */
 Lisp_Object Vcommand_line_args;
 
+/* The name under which Emacs was invoked, with any leading directory
+   names discarded.  */
+Lisp_Object Vinvocation_name;
+
+/* The directory name from which Emacs was invoked.  */
+Lisp_Object Vinvocation_directory;
+
+/* The directory name in which to find subdirs such as lisp and etc.
+   nil means get them only from PATH_LOADSEARCH.  */
+Lisp_Object Vinstallation_directory;
+
 /* Hook run by `kill-emacs' before it does really anything.  */
 Lisp_Object Vkill_emacs_hook;
 
@@ -146,23 +134,41 @@ Lisp_Object Vkill_emacs_hook;
   on subsequent starts.  */
 int initialized;
 
-/* Variable whose value is symbol giving operating system type */
+/* Variable whose value is symbol giving operating system type.  */
 Lisp_Object Vsystem_type;
+
+/* Variable whose value is string giving configuration built for.  */
+Lisp_Object Vsystem_configuration;
   
 /* If non-zero, emacs should not attempt to use an window-specific code,
    but instead should use the virtual terminal under which it was started */
 /* 92.3.31 by K.Handa - If negative, Mule use Sun console feature. */
 int inhibit_window_system;
 
+/* If nonzero, set Emacs to run at this priority.  This is also used
+   in child_setup and sys_suspend to make sure subshells run at normal
+   priority; Those functions have their own extern declaration.  */
+int emacs_priority;
+
+#ifdef BSD_PGRPS
+/* See sysdep.c.  */
+extern int inherited_pgroup;
+#endif
+
 /* 92.3.31 by K.Handa, 92.10.21 by M.Higashida  */
 #if defined (HAVE_X_WINDOWS) || defined (HAVE_SUN_CONSOLE) || (defined (MSDOS) && defined(HAVE_VGA_ADAPTER)) || defined (WIN32)
-/* If -d option is used, this variable points to the name of
-   the display to use.  */
-char *alternate_display;
-char **xargv;
-int xargc;
+/* If non-zero, -d was specified, meaning we're using some window system. */
+int display_arg;
 #endif /* HAVE_X_WINDOWS or HAVE_SUN_CONSOLE or (MSDOS and HAVE_VGA_ADAPTER) or WIN32 */
 
+/* An address near the bottom of the stack.
+   Tells GC how to save a copy of the stack.  */
+char *stack_bottom;
+
+#ifdef HAVE_X_WINDOWS
+extern Lisp_Object Vwindow_system;
+#endif /* HAVE_X_WINDOWS */
+
 #ifdef USG_SHARED_LIBRARIES
 /* If nonzero, this is the place to put the end of the writable segment
    at startup.  */
@@ -179,6 +185,10 @@ int noninteractive;
    but nothing terrible happens if user sets this one.  */
 
 int noninteractive1;
+
+/* Save argv and argc.  */
+char **initial_argv;
+int initial_argc;
 
 /* Signal code for the fatal signal that was received */
 int fatal_error_code;
@@ -187,51 +197,50 @@ int fatal_error_code;
 int fatal_error_in_progress;
 
 /* Handle bus errors, illegal instruction, etc. */
+SIGTYPE
 fatal_error_signal (sig)
      int sig;
 {
-#ifdef BSD
-  int tpgrp;
-#endif /* BSD */
-
   fatal_error_code = sig;
   signal (sig, SIG_DFL);
 
   /* If fatal error occurs in code below, avoid infinite recursion.  */
-  if (fatal_error_in_progress)
-    kill (getpid (), fatal_error_code);
-
-  fatal_error_in_progress = 1;
-
-  /* If we are controlling the terminal, reset terminal modes */
-#ifdef BSD
-  if (ioctl(0, TIOCGPGRP, &tpgrp) == 0
-      && tpgrp == getpgrp (0))
-#endif /* BSD */
+  if (! fatal_error_in_progress)
     {
-      reset_sys_modes ();
-      if (sig != SIGTERM)
-	fprintf (stderr, "Fatal error (%d).", sig);
+      fatal_error_in_progress = 1;
+
+      shut_down_emacs (sig, 0, Qnil);
     }
 
-  /* Clean up */
-#ifdef subprocesses
-  kill_buffer_processes (Qnil);
-#endif
-  Fdo_auto_save (Qt);
-
-#ifdef CLASH_DETECTION
-  unlock_all_files ();
-#endif /* CLASH_DETECTION */
-
 #ifdef VMS
-  kill_vms_processes ();
   LIB$STOP (SS$_ABORT);
 #else
-  /* Signal the same code; this time it will really be fatal.  */
+  /* Signal the same code; this time it will really be fatal.
+     Remember that since we're in a signal handler, the signal we're
+     going to send is probably blocked, so we have to unblock it if we
+     want to really receive it.  */
+#ifndef MSDOS
+  sigunblock (sigmask (fatal_error_code));
+#endif
   kill (getpid (), fatal_error_code);
 #endif /* not VMS */
 }
+
+#ifdef SIGDANGER
+
+/* Handler for SIGDANGER.  */
+SIGTYPE
+memory_warning_signal (sig)
+     int sig;
+{
+  signal (sig, memory_warning_signal);
+
+  malloc_warning ("Operating system warns that virtual memory is running low.\n");
+
+  /* It might be unsafe to call do_auto_save now.  */
+  force_auto_save_soon ();
+}
+#endif
 
 /* Code for dealing with Lisp access to the Unix command line */
 
@@ -242,6 +251,80 @@ init_cmdargs (argc, argv, skip_args)
      int skip_args;
 {
   register int i;
+  Lisp_Object name, dir;
+
+  initial_argv = argv;
+  initial_argc = argc;
+
+  Vinvocation_name = Ffile_name_nondirectory (build_string (argv[0]));
+  Vinvocation_directory = Ffile_name_directory (build_string (argv[0]));
+  /* If we got no directory in argv[0], search PATH to find where
+     Emacs actually came from.  */
+  if (NILP (Vinvocation_directory))
+    {
+      Lisp_Object found;
+      int yes = openp (Vexec_path, Vinvocation_name,
+		       EXEC_SUFFIXES, &found, 1);
+      if (yes == 1)
+	Vinvocation_directory = Ffile_name_directory (found);
+    }
+
+  Vinstallation_directory = Qnil;
+
+  if (!NILP (Vinvocation_directory))
+    {
+      dir = Vinvocation_directory;
+      name = Fexpand_file_name (Vinvocation_name, dir);
+      while (1)
+	{
+	  Lisp_Object tem, lib_src_exists;
+	  Lisp_Object etc_exists, info_exists;
+
+	  /* See if dir contains subdirs for use by Emacs.
+	     Check for the ones that would exist in a build directory,
+	     not including lisp and info.  */
+	  tem = Fexpand_file_name (build_string ("lib-src"), dir);
+	  lib_src_exists = Ffile_exists_p (tem);
+	  if (!NILP (lib_src_exists))
+	    {
+	      tem = Fexpand_file_name (build_string ("etc"), dir);
+	      etc_exists = Ffile_exists_p (tem);
+	      if (!NILP (etc_exists))
+		{
+		  Vinstallation_directory
+		    = Ffile_name_as_directory (dir);
+		  break;
+		}
+	    }
+
+	  /* See if dir's parent contains those subdirs.  */
+	  tem = Fexpand_file_name (build_string ("../lib-src"), dir);
+	  lib_src_exists = Ffile_exists_p (tem);
+	  if (!NILP (lib_src_exists))
+	    {
+	      tem = Fexpand_file_name (build_string ("../etc"), dir);
+	      etc_exists = Ffile_exists_p (tem);
+	      if (!NILP (etc_exists))
+		{
+		  tem = Fexpand_file_name (build_string (".."), dir);
+		  Vinstallation_directory
+		    = Ffile_name_as_directory (tem);
+		  break;
+		}
+	    }
+
+	  /* If the Emacs executable is actually a link,
+	     next try the dir that the link points into.  */
+	  tem = Ffile_symlink_p (name);
+	  if (!NILP (tem))
+	    {
+	      name = Fexpand_file_name (tem, dir);
+	      dir = Ffile_name_directory (name);
+	    }
+	  else
+	    break;
+	}
+    }
 
   Vcommand_line_args = Qnil;
 
@@ -252,16 +335,28 @@ init_cmdargs (argc, argv, skip_args)
 	  = Fcons (build_string (argv[i]), Vcommand_line_args);
     }
 }
+
+DEFUN ("invocation-name", Finvocation_name, Sinvocation_name, 0, 0, 0,
+  "Return the program name that was used to run Emacs.\n\
+Any directory names are omitted.")
+  ()
+{
+  return Fcopy_sequence (Vinvocation_name);
+}
+
+DEFUN ("invocation-directory", Finvocation_directory, Sinvocation_directory,
+  0, 0, 0,
+  "Return the directory name in which the Emacs executable was located")
+  ()
+{
+  return Fcopy_sequence (Vinvocation_directory);
+}
+
 
 #ifdef VMS
 #ifdef LINK_CRTL_SHARE
 #ifdef SHAREABLE_LIB_BUG
-#ifdef __GNUC__
-#define	environ $$PsectAttributes_NOSHR$$environ
-extern char **environ;
-#else
 extern noshare char **environ;
-#endif
 #endif /* SHAREABLE_LIB_BUG */
 #endif /* LINK_CRTL_SHARE */
 #endif /* VMS */
@@ -285,24 +380,29 @@ asm ("	.stabs	\"___CTOR_LIST__\", 23, 0, 0, L_setbinmode");
 #endif
 /* end of patch */
 
+#ifndef ORDINARY_LINK
 /* We don't include crtbegin.o and crtend.o in the link,
    so these functions and variables might be missed.
    Provide dummy definitions to avoid error.
    (We don't have any real constructors or destructors.)  */
 #ifdef __GNUC__
-#ifndef ORDINARY_LINK
-__do_clobal_ctors ()
+#ifndef GCC_CTORS_IN_LIBC
+__do_global_ctors ()
 {}
-__do_clobal_ctors_aux ()
+__do_global_ctors_aux ()
 {}
 __do_global_dtors ()
 {}
+/* Linux has a bug in its library; avoid an error.  */
+#ifndef LINUX
 char * __CTOR_LIST__[2] = { (char *) (-1), 0 };
+#endif
 char * __DTOR_LIST__[2] = { (char *) (-1), 0 };
+#endif /* GCC_CTORS_IN_LIBC */
 __main ()
 {}
-#endif /* not ORDINARY_LINK */
 #endif /* __GNUC__ */
+#endif /* ORDINARY_LINK */
 
 #ifdef NeXT
 #include <mach/mach.h>
@@ -315,9 +415,10 @@ main (argc, argv, envp)
      char **argv;
      char **envp;
 {
+  char stack_bottom_variable;
   int skip_args = 0;
   extern int errno;
-  extern void malloc_warning ();
+  extern sys_nerr;
 
 /* Map in shared memory, if we are using that.  */
 #ifdef HAVE_SHM
@@ -335,6 +436,37 @@ main (argc, argv, envp)
     }
 #endif
 
+#ifdef NeXT
+  extern int malloc_cookie;
+
+  /* This helps out unexnext.c.  */
+  if (initialized)
+    {
+      if (malloc_jumpstart (malloc_cookie) != 0)
+	fatal("malloc jumpstart failed!\n");
+    }
+  else
+    {
+      emacszone = NXCreateZone(ZONESIZE*vm_page_size,vm_page_size,1);
+      if(emacszone == NX_NOZONE)
+	fatal("can't create emacs zone.\n");
+    }
+#endif /* NeXT */
+
+#ifdef HAVE_X_WINDOWS
+  /* Stupid kludge to catch command-line display spec.  We can't
+     handle this argument entirely in window system dependent code
+     because we don't even know which window system dependent code
+     to run until we've recognized this argument.  */
+  {
+    int i;
+
+    for (i = 1; (i < argc && ! display_arg); i++)
+      if (!strcmp (argv[i], "-d") || !strcmp (argv[i], "-display"))
+	display_arg = 1;
+  }
+#endif
+
 #ifdef VMS
   /* If -map specified, map the data file in */
   if (argc > 2 && ! strcmp (argv[1], "-map"))
@@ -366,25 +498,19 @@ main (argc, argv, envp)
 #endif
 /* end of patch */
 
+  /* Record (approximately) where the stack begins.  */
+  stack_bottom = &stack_bottom_variable;
+
+#ifdef RUN_TIME_REMAP
+  if (initialized)
+    run_time_remap (argv[0]);
+#endif
+
 #ifdef USG_SHARED_LIBRARIES
   if (bss_end)
     brk (bss_end);
 #endif
 
-#ifdef NeXT
-  extern int malloc_cookie;
-
-  /* This helps out unexnext.c.  */
-  if (initialized) {
-    if (malloc_jumpstart (malloc_cookie) != 0)
-      fatal("malloc jumpstart failed!\n");
-  } else {
-    emacszone = NXCreateZone(ZONESIZE*vm_page_size,vm_page_size,1);
-    if(emacszone == NX_NOZONE)
-      fatal("can't create emacs zone.\n");
-  }
-#endif /* NeXT */
-
   clearerr (stdin);
 
 #ifdef APOLLO
@@ -397,87 +523,65 @@ main (argc, argv, envp)
 #endif /* APOLLO */
 
 #ifndef SYSTEM_MALLOC
-  /* Arrange for warnings when nearly out of space.  */
-  malloc_init (0, malloc_warning);
-#endif
+  if (! initialized)
+    {
+      /* Arrange to get warning messages as memory fills up.  */
+      memory_warnings (0, malloc_warning);
 
-/* 91.10.16 by S.Hirano */
-#if defined(MSDOS) && defined(GO32)
-  _fmode = O_BINARY;	/* all of files are treated as binary files */
-  (stdin)->_flag &= ~_IOTEXT;   /* also binary for stdio */
+      /* Arrange to disable interrupt input while malloc and friends are
+	 running.  */
+      uninterrupt_malloc ();
+    }
+#endif	/* not SYSTEM_MALLOC */
+
+#ifdef MSDOS
+  /* We do all file input/output as binary files.  When we need to translate
+     newlines, we do that manually.  */
+  _fmode = O_BINARY;
+  (stdin)->_flag &= ~_IOTEXT;
   (stdout)->_flag &= ~_IOTEXT;
   (stderr)->_flag &= ~_IOTEXT;
 #endif /* MSDOS */
-/* end of patch */
 
-#ifdef HIGHPRI
-  setpriority (PRIO_PROCESS, getpid (), HIGHPRI);
+#ifdef SET_EMACS_PRIORITY
+  if (emacs_priority)
+    nice (emacs_priority);
   setuid (getuid ());
-#endif HIGHPRI
+#endif /* SET_EMACS_PRIORITY */
+
+#ifdef EXTRA_INITIALIZE
+  EXTRA_INITIALIZE;
+#endif
 
   inhibit_window_system = 0;
 
-/* 92.3.31 by K.Handa, 92.10.21 by M.Higashida */
-#if defined (HAVE_X_WINDOWS) || defined (HAVE_SUN_CONSOLE) || (defined(MSDOS) && defined(HAVE_VGA_ADAPTER)) || defined (WIN32)
-  xargv = argv;
-  xargc = argc;
-#endif
-
-/* Handle the -t switch, which specifies filename to use as terminal */
+  /* Handle the -t switch, which specifies filename to use as terminal */
   if (skip_args + 2 < argc && !strcmp (argv[skip_args + 1], "-t"))
     {
+      int result;
       skip_args += 2;
       close (0);
       close (1);
-      open (argv[skip_args], O_RDWR, 2 );
+      result = open (argv[skip_args], O_RDWR, 2 );
+      if (result < 0)
+	{
+	  char *errstring = strerror (errno);
+	  fprintf (stderr, "emacs: %s: %s\n", argv[skip_args], errstring);
+	  exit (1);
+	}
       dup (0);
+      if (! isatty (0))
+	{
+	  fprintf (stderr, "emacs: %s: not a tty\n", argv[skip_args]);
+	  exit (1);
+	}
       fprintf (stderr, "Using %s\n", argv[skip_args]);
 /* 92.3.31 by K.Handa, 92.10.21 by M.Higashida */
 #if defined (HAVE_X_WINDOWS) || defined (HAVE_SUN_CONSOLE) || (defined(MSDOS) && defined(HAVE_VGA_ADAPTER)) || defined (WIN32)
       inhibit_window_system = 1;	/* -t => -nw */
 #endif
     }
-#ifdef HAVE_X_WINDOWS
-/* Handle the -d switch, which means use a different display for X */
-  if (skip_args + 2 < argc && (!strcmp (argv[skip_args + 1], "-d") ||
-			       !strcmp (argv[skip_args + 1], "-display")))
-    {
-      skip_args += 2;
-      alternate_display = argv[skip_args];
-    } 
-  else
-    alternate_display = 0;
-#endif	/* HAVE_X_WINDOWS */
-/* 92.3.31 by K.Handa */
-#ifdef HAVE_SUN_CONSOLE
-  if (skip_args + 1 < argc && (!strcmp (argv[skip_args + 1], "-sun"))) {
-    extern Pixrect *screen;
 
-    inhibit_window_system = -1;
-    skip_args++;
-    alternate_display = "/dev/fb";
-    if (skip_args + 2 < argc && (!strcmp (argv[skip_args + 1], "-fb"))) {
-      skip_args += 2;
-      alternate_display = argv[skip_args];
-    }
-    if (screen = pr_open(alternate_display)) /* Just a test. */
-      pr_close(screen);
-    else
-      alternate_display = 0;
-  }
-#endif /* HAVE_SUN_CONSOLE */
-/* end of patch */
-/* 92.10.17, 93.2.17 by M.Higashida */
-#if defined(MSDOS) && defined(HAVE_VGA_ADAPTER)
-  if (skip_args + 1 < argc && (!strcmp (argv[skip_args + 1], "-vga"))) {
-    inhibit_window_system = -1;
-    skip_args++;
-  }
-#endif /* MSDOS and HAVE_VGA_ADAPTER */
-#ifdef WIN32
-  inhibit_window_system = -1;
-#endif
-/* end of patch */
   if (skip_args + 1 < argc
       && (!strcmp (argv[skip_args + 1], "-nw")))
     {
@@ -493,19 +597,25 @@ main (argc, argv, envp)
       noninteractive = 1;
     }
 
+  if (! noninteractive)
+    {
+#ifdef BSD_PGRPS
+      if (initialized)
+	{
+	  inherited_pgroup = EMACS_GETPGRP (0);
+	  setpgrp (0, getpid ());
+	}
+#else
+#if defined (USG5) && defined (INTERRUPT_INPUT)
+      setpgrp ();
+#endif
+#endif
+    }
+
 #ifdef POSIX_SIGNALS
   init_signals ();
 #endif
 
-#ifdef HAVE_TZSET
-  /* Reinitialize the time zone if it was initialized before dumping Emacs.  */
-  if (initialized)
-    tzset ();
-#endif
-#ifdef WIN32 /* 93.2.25 by M.Higashida */
-  _tzset ();
-#endif
-
   if (
 #ifndef CANNOT_DUMP
       ! noninteractive || initialized
@@ -517,58 +627,43 @@ main (argc, argv, envp)
       /* Don't catch these signals in batch mode if not initialized.
 	 On some machines, this sets static data that would make
 	 signal fail to work right when the dumped Emacs is run.  */
-/* 91.10.16 by S.Hirano, 92.11.4 by M.Higashida */
-#ifdef MSDOS
-#ifdef GO32
-      /* Nothing */
-#else
-#ifdef EMX
-      signal (SIGBREAK, fatal_error_signal);
-      signal (SIGQUIT, fatal_error_signal);
-#endif
-#endif
-#else /* not MSDOS */
       signal (SIGHUP, fatal_error_signal);
       signal (SIGQUIT, fatal_error_signal);
       signal (SIGILL, fatal_error_signal);
       signal (SIGTRAP, fatal_error_signal);
+#ifdef SIGIOT
+      /* This is missing on some systems - OS/2, for example.  */
       signal (SIGIOT, fatal_error_signal);
+#endif
 #ifdef SIGEMT
       signal (SIGEMT, fatal_error_signal);
 #endif
       signal (SIGFPE, fatal_error_signal);
+#ifdef SIGBUS
       signal (SIGBUS, fatal_error_signal);
+#endif
       signal (SIGSEGV, fatal_error_signal);
+#ifdef SIGSYS
       signal (SIGSYS, fatal_error_signal);
+#endif
       signal (SIGTERM, fatal_error_signal);
 #ifdef SIGXCPU
       signal (SIGXCPU, fatal_error_signal);
 #endif
 #ifdef SIGXFSZ
       signal (SIGXFSZ, fatal_error_signal);
-#endif SIGXFSZ
-#endif /* not MSDOS and GO32 */
-/* end of patch */
+#endif /* SIGXFSZ */
+
+#ifdef SIGDANGER
+      /* This just means available memory is getting low.  */
+      signal (SIGDANGER, memory_warning_signal);
+#endif
 
 #ifdef AIX
-      /* This used to run fatal_error_signal,
-	 but it isn't fatal.  There's nothing Emacs can usefully do.
-	 Might as well let the system kill us if it insists.  */
-      signal (SIGDANGER, SIG_IGN);
-      signal (20, fatal_error_signal);
-      signal (21, fatal_error_signal);
-      signal (22, fatal_error_signal);
-      signal (23, fatal_error_signal);
-      signal (24, fatal_error_signal);
-#ifdef SIGIO
-      signal (SIGAIO, fatal_error_signal);
-      signal (SIGPTY, fatal_error_signal);
-#endif
-#ifdef SIGURG
-      /* Note that SIGIOINT is the same as SIGIO on some machines,
-	 and the same as SIGURG on others.  It seems ore reliable to use the
-	 name with a uniform meaning.  */
-      signal (SIGURG, fatal_error_signal);
+/* 20 is SIGCHLD, 21 is SIGTTIN, 22 is SIGTTOU.  */
+      signal (SIGXCPU, fatal_error_signal);
+#ifndef _I386
+      signal (SIGIOINT, fatal_error_signal);
 #endif
       signal (SIGGRANT, fatal_error_signal);
       signal (SIGRETRACT, fatal_error_signal);
@@ -586,10 +681,11 @@ main (argc, argv, envp)
       init_alloc_once ();
       init_obarray ();
       init_eval_once ();
-      init_mc_once ();		/* 89.7.26, 91.10.19 by K.Handa */
+      init_charset_once ();	/* 89.7.26, 91.10.19 by K.Handa */
       init_syntax_once ();	/* Create standard syntax table.  */
       init_category_once ();	/* 91.12.7 by K.Handa */
 		      /* Must be done before init_buffer */
+      init_casetab_once ();
       init_buffer_once ();	/* Create buffer table and some buffers */
       init_minibuf_once ();	/* Create list of minibuffers */
 			      /* Must precede init_window_once */
@@ -602,42 +698,63 @@ main (argc, argv, envp)
     }
 
   init_alloc ();
-#ifdef MAINTAIN_ENVIRONMENT
-  init_environ ();
-/* 92.3.25 by N.Hikichi */
-#ifdef HAVE_X_WINDOWS
-  if (alternate_display)
-    set_environment_alist (build_string ("DISPLAY"),
-			   build_string (alternate_display));
-#endif /* HAVE_X_WINDOWS */
-/* end of patch */
-#endif
-  init_codeconv ();		/* 92.1.7 by K.Handa */
+  init_coding ();		/* 92.1.7 by K.Handa */
   init_eval ();
   init_data ();
-  init_read ();
+#ifdef MULE			/* 94.5.25 by K.Handa */
+  init_fontset ();
+#endif  /* MULE */
+
+#ifdef MSDOS
+  /* Call early 'cause init_environment needs it.  */
+  init_dosfns ();
+  /* Set defaults for several environment variables.  */
+  if (initialized) init_environment (argc, argv, skip_args);
+#endif
+
+  /* egetenv is a pretty low-level facility, which may get called in
+     many circumstances; it seems flimsy to put off initializing it
+     until calling init_callproc.  */
+  set_process_environment ();
+  /* AIX crashes are reported in system versions 3.2.3 and 3.2.4
+     if this is not done.  Do it after set_process_environment so that we
+     don't pollute Vprocess_environment.  */
+#ifdef AIX
+  putenv ("LANG=C");
+#endif
 
-  init_cmdargs (argc, argv, skip_args);	/* Create list Vcommand_line_args */
   init_buffer ();	/* Init default directory of main buffer */
+
+  init_callproc_1 ();	/* Must precede init_cmdargs and init_sys_modes.  */
+  init_cmdargs (argc, argv, skip_args);	/* Must precede init_lread.  */
+  init_callproc ();	/* Must follow init_cmdargs but not init_sys_modes.  */
+  init_lread ();
+
   if (!noninteractive)
     {
 #ifdef VMS
-      init_vms_input ();/* init_display calls get_screen_size, that needs this */
+      init_vms_input ();/* init_display calls get_frame_size, that needs this */
 #endif /* VMS */
       init_display ();	/* Determine terminal type.  init_sys_modes uses results */
     }
   init_keyboard ();	/* This too must precede init_sys_modes */
-  init_callproc ();	/* And this too. */
+#ifdef VMS
+  init_vmsproc ();	/* And this too. */
+#endif /* VMS */
   init_sys_modes ();	/* Init system terminal modes (RAW or CBREAK, etc.) */
   init_xdisp ();
   init_macros ();
   init_editfns ();
+#ifdef LISP_FLOAT_TYPE
+  init_floatfns ();
+#endif
 #ifdef VMS
   init_vmsfns ();
 #endif /* VMS */
-#ifdef subprocesses
   init_process ();
-#endif /* subprocesses */
+#ifdef CLASH_DETECTION
+  init_filelock ();
+#endif /* CLASH_DETECTION */
 
 /* Intern the names of all standard functions and variables; define standard keys */
 
@@ -648,22 +765,21 @@ main (argc, argv, envp)
 	 for the sake of symbols like error-message */
       syms_of_data ();
       syms_of_alloc ();
-#ifdef MAINTAIN_ENVIRONMENT
-      syms_of_environ ();
-#endif MAINTAIN_ENVIRONMENT
-      syms_of_read ();
+      syms_of_lread ();
       syms_of_print ();
       syms_of_eval ();
       syms_of_fns ();
+      syms_of_floatfns ();
 
       syms_of_abbrev ();
       syms_of_buffer ();
       syms_of_bytecode ();
       syms_of_callint ();
       syms_of_casefiddle ();
+      syms_of_casetab ();
       syms_of_callproc ();
       syms_of_category ();	/* 91.11.11 by K.Handa */
-      syms_of_codeconv ();	/* 92.1.7 by K.Handa */
+      syms_of_coding ();	/* 92.1.7 by K.Handa */
       syms_of_cmds ();
 #ifndef NO_DIR_LIBRARY
       syms_of_dired ();
@@ -676,6 +792,9 @@ main (argc, argv, envp)
 #ifdef CLASH_DETECTION
       syms_of_filelock ();
 #endif /* CLASH_DETECTION */
+#ifdef MULE			/* 94.5.25 by K.Handa */
+      syms_of_fontset ();
+#endif  /* MULE */
 /* 92.11.1 by M.Higashida */
 #ifdef FILE_TRANSLATION_MODE
       syms_of_transmode ();
@@ -688,20 +807,35 @@ main (argc, argv, envp)
       syms_of_marker ();
       syms_of_minibuf ();
       syms_of_mocklisp ();
-#ifdef subprocesses
       syms_of_process ();
-#endif /* subprocesses */
       syms_of_search ();
+      syms_of_frame ();
       syms_of_syntax ();
+      syms_of_term ();
       syms_of_undo ();
+
+      /* Only defined if Emacs is compiled with USE_TEXT_PROPERTIES */
+      syms_of_textprop ();
+#ifdef VMS
+      syms_of_vmsproc ();
+#endif /* VMS */
       syms_of_window ();
       syms_of_xdisp ();
 #ifdef HAVE_X_WINDOWS
+      syms_of_xterm ();
       syms_of_xfns ();
+#ifdef HAVE_X11
+      syms_of_xselect ();
+#endif
 #ifdef HAVE_X_MENU
       syms_of_xmenu ();
 #endif /* HAVE_X_MENU */
 #endif /* HAVE_X_WINDOWS */
+
+#if defined (HAVE_X_WINDOWS) || defined (TERMINAL_FACE)
+      syms_of_xfaces ();
+#endif /* HAVE_X_WINDOWS or TERMINAL_FACE */
+
 /* 87.6.8, 91.11.2, 92.3.31, 92.7.31  by K.Handa, 92.10.17, 93.2.17 by M.Higashida */
 #ifdef HAVE_SUN_CONSOLE
       syms_of_sunterm ();
@@ -712,15 +846,15 @@ main (argc, argv, envp)
 #ifdef WIN32
       syms_of_win32term ();
 #endif
-      syms_of_mc ();
+      syms_of_mule ();		/* 94.7.15 by K.Handa */
+      syms_of_charset ();
       syms_of_ccl ();		/* 93.5.14 by K.Handa */
 #ifdef WNN4
       syms_of_wnn ();
-#else  /* not WNN4 */
+#endif
 #ifdef CANNA
       syms_of_canna ();
 #endif /* CANNA */
-#endif /* not WNN4 */
 /* end of patch */
 
 #ifdef MCPATH
@@ -743,6 +877,7 @@ main (argc, argv, envp)
       keys_of_macros ();
       keys_of_minibuf ();
       keys_of_window ();
+      keys_of_frame ();
     }
 
   if (!initialized)
@@ -761,60 +896,44 @@ main (argc, argv, envp)
 
   initialized = 1;
 
+#if defined (sun) || defined (LOCALTIME_CACHE)
+  /* sun's localtime has a bug.  it caches the value of the time
+     zone rather than looking it up every time.  Since localtime() is
+     called to bolt the undumping time into the undumped emacs, this
+     results in localtime ignoring the TZ environment variable.
+     This flushes the new TZ value into localtime. */
+  tzset ();
+#endif /* defined (sun) || defined (LOCALTIME_CACHE) */
+
   /* Enter editor command loop.  This never returns.  */
   Frecursive_edit ();
   /* NOTREACHED */
 }
 
 DEFUN ("kill-emacs", Fkill_emacs, Skill_emacs, 0, 1, "P",
-  "Exit the Emacs job and kill it.  ARG means no query.\n\
-If emacs is running noninteractively and ARG is an integer,\n\
-return ARG as the exit program code.")
+  "Exit the Emacs job and kill it.\n\
+If ARG is an integer, return ARG as the exit program code.\n\
+If ARG is a  string, stuff it as keyboard input.\n\n\
+The value of `kill-emacs-hook', if not void,\n\
+is a list of functions (of no args),\n\
+all of which are called before Emacs is actually killed.")
   (arg)
      Lisp_Object arg;
 {
-  Lisp_Object answer;
+  Lisp_Object hook, hook1;
   int i;
   struct gcpro gcpro1;
 
   GCPRO1 (arg);
 
-  if (!EQ (Vkill_emacs_hook, Qnil))
-    call0 (Vkill_emacs_hook);
-
   if (feof (stdin))
     arg = Qt;
 
-#ifdef EGG
-#ifdef WNN3
-      wnn_exit();
-#else  /* not WNN3 */
-#ifdef WNN4V3
-      wnn_exit();
-#else  /* not WNN4V3 */
-#ifdef WNN4
-      wnn_exit();
-#endif /* WNN4 */
-#endif /* not WNN4V3 */
-#endif /* not WNN3 */
-#endif /* EGG */
+/* 93.9.25 by T.Shingu -- Now wnn wnn_exit() is done in wnn4-egg.el.*/
 
-#ifdef subprocesses
-  kill_buffer_processes (Qnil);
-#endif /* subprocesses */
+  if (!NILP (Vrun_hooks) && !noninteractive)
+    call1 (Vrun_hooks, intern ("kill-emacs-hook"));
 
-#ifdef VMS
-  kill_vms_processes ();
-#endif /* VMS */
-
-  Fdo_auto_save (Qt);
-
-#ifdef CLASH_DETECTION
-  unlock_all_files ();
-#endif /* CLASH_DETECTION */
-
-  fflush (stdout);
-  reset_sys_modes ();
   UNGCPRO;
 
 /* Is it really necessary to do this deassign
@@ -822,13 +941,9 @@ return ARG as the exit program code.")
 /* #ifdef VMS
   stop_vms_input ();
  #endif  */
-  stuff_buffered_input (arg);
-#ifdef SIGIO
-  /* There is a tendency for a SIGIO signal to arrive within exit,
-     and cause a SIGHUP because the input descriptor is already closed.  */
-  unrequest_sigio ();
-  signal (SIGIO, SIG_IGN);
-#endif
+
+  shut_down_emacs (0, 0, STRINGP (arg) ? arg : Qnil);
+
   exit ((XTYPE (arg) == Lisp_Int) ? XINT (arg)
 #ifdef VMS
 	: 1
@@ -838,6 +953,80 @@ return ARG as the exit program code.")
 	);
   /* NOTREACHED */
 }
+
+
+/* Perform an orderly shutdown of Emacs.  Autosave any modified
+   buffers, kill any child processes, clean up the terminal modes (if
+   we're in the foreground), and other stuff like that.  Don't perform
+   any redisplay; this may be called when Emacs is shutting down in
+   the background, or after its X connection has died.
+
+   If SIG is a signal number, print a message for it.
+
+   This is called by fatal signal handlers, X protocol error handlers,
+   and Fkill_emacs.  */
+
+void
+shut_down_emacs (sig, no_x, stuff)
+     int sig, no_x;
+     Lisp_Object stuff;
+{
+  /* Prevent running of hooks from now on.  */
+  Vrun_hooks = Qnil;
+
+  /* If we are controlling the terminal, reset terminal modes */
+#ifdef EMACS_HAVE_TTY_PGRP
+  {
+    int pgrp = EMACS_GETPGRP (0);
+
+    int tpgrp;
+    if (EMACS_GET_TTY_PGRP (0, &tpgrp) != -1
+	&& tpgrp == pgrp)
+      {
+	fflush (stdout);
+	reset_sys_modes ();
+	if (sig && sig != SIGTERM)
+	  fprintf (stderr, "Fatal error (%d).", sig);
+      }
+  }
+#else
+  fflush (stdout);
+  reset_sys_modes ();
+#endif
+
+  stuff_buffered_input (stuff);
+
+  kill_buffer_processes (Qnil);
+  Fdo_auto_save (Qt, Qnil);
+
+#ifdef CLASH_DETECTION
+  unlock_all_files ();
+#endif
+
+#ifdef VMS
+  kill_vms_processes ();
+#endif
+
+#if 0 /* This triggers a bug in XCloseDisplay and is not needed.  */
+#ifdef HAVE_X_WINDOWS
+  /* It's not safe to call intern here.  Maybe we are crashing.  */
+  if (!noninteractive && SYMBOLP (Vwindow_system)
+      && XSYMBOL (Vwindow_system)->name->size == 1
+      && XSYMBOL (Vwindow_system)->name->data[0] == 'x'
+      && ! no_x)
+    Fx_close_current_connection ();
+#endif /* HAVE_X_WINDOWS */
+#endif
+
+#ifdef SIGIO
+  /* There is a tendency for a SIGIO signal to arrive within exit,
+     and cause a SIGHUP because the input descriptor is already closed.  */
+  unrequest_sigio ();
+  signal (SIGIO, SIG_IGN);
+#endif
+}
+
+
 
 #ifndef CANNOT_DUMP
 /* Nothing like this can be implemented on an Apollo.
@@ -853,7 +1042,6 @@ This function exists on systems that use HAVE_SHM.")
 {
   extern int my_edata;
   Lisp_Object tem;
-  extern void malloc_warning ();
 
   CHECK_STRING (intoname, 0);
   intoname = Fexpand_file_name (intoname, Qnil);
@@ -865,7 +1053,7 @@ This function exists on systems that use HAVE_SHM.")
   /* Tell malloc where start of impure now is */
   /* Also arrange for warnings when nearly out of space.  */
 #ifndef SYSTEM_MALLOC
-  malloc_init (&my_edata, malloc_warning);
+  memory_warnings (&my_edata, malloc_warning);
 #endif
   map_out_data (XSTRING (intoname)->data);
 
@@ -878,17 +1066,21 @@ This function exists on systems that use HAVE_SHM.")
 
 DEFUN ("dump-emacs", Fdump_emacs, Sdump_emacs, 2, 2, 0,
   "Dump current state of Emacs into executable file FILENAME.\n\
-Take symbols from SYMFILE (presumably the file you executed to run Emacs).")
+Take symbols from SYMFILE (presumably the file you executed to run Emacs).\n\
+This is used in the file `loadup.el' when building Emacs.\n\
+\n\
+Bind `command-line-processed' to nil before dumping,\n\
+if you want the dumped Emacs to process its command line\n\
+and announce itself normally when it is run.")
   (intoname, symname)
      Lisp_Object intoname, symname;
 {
   extern int my_edata;
   Lisp_Object tem;
-  extern void malloc_warning ();
 
   CHECK_STRING (intoname, 0);
   intoname = Fexpand_file_name (intoname, Qnil);
-  if (!EQ (symname, Qnil))
+  if (!NILP (symname))
     {
       CHECK_STRING (symname, 0);
       if (XSTRING (symname)->size)
@@ -905,7 +1097,7 @@ Take symbols from SYMFILE (presumably the file you executed to run Emacs).")
   /* Tell malloc where start of impure now is */
   /* Also arrange for warnings when nearly out of space.  */
 #ifndef SYSTEM_MALLOC
-  malloc_init (&my_edata, malloc_warning);
+  memory_warnings (&my_edata, malloc_warning);
 #endif
 /* 92.11.4 by M.Higashida */
 #if defined(MSDOS) && defined(EMX)
@@ -921,7 +1113,7 @@ Take symbols from SYMFILE (presumably the file you executed to run Emacs).")
   }
 #else /* not MSDOS and EMX */
   unexec (XSTRING (intoname)->data,
-	  !EQ (symname, Qnil) ? XSTRING (symname)->data : 0, &my_edata, 0, 0);
+	  !NILP (symname) ? XSTRING (symname)->data : 0, &my_edata, 0, 0);
 #endif /* not MSDOS and EMX */
 /* end of patch */
 #endif /* not VMS */
@@ -935,16 +1127,8 @@ Take symbols from SYMFILE (presumably the file you executed to run Emacs).")
 
 #endif /* not CANNOT_DUMP */
 
-#ifdef VMS
-#define SEPCHAR ','
-#else
-/* 91.10.16 by S.Hirano, 93.2.17 by M.Higashida */
-#if defined(MSDOS) || defined(WIN32)
-#define SEPCHAR ';'
-#else
+#ifndef SEPCHAR
 #define SEPCHAR ':'
-#endif /* MSDOS or WIN32 */
-/* end of patch */
 #endif
 
 Lisp_Object
@@ -952,21 +1136,25 @@ decode_env_path (evarname, defalt)
      char *evarname, *defalt;
 {
   register char *path, *p;
-  extern char *index ();
 
   Lisp_Object lpath;
 
+  /* It's okay to use getenv here, because this function is only used
+     to initialize variables when Emacs starts up, and isn't called
+     after that.  */
   if (evarname != 0)
-    path = (char *) egetenv (evarname);
+    path = (char *) getenv (evarname);
   else
     path = 0;
   if (!path)
     path = defalt;
+
 /* 91.10.16 by S.Hirano, 93.2.17 by M.Higashida */
 #if defined(MSDOS) || defined(WIN32)
   dostounix_filename(path);
 #endif
 /* end of patch */
+
   lpath = Qnil;
   while (1)
     {
@@ -990,23 +1178,55 @@ syms_of_emacs ()
 #else
   defsubr (&Sdump_emacs);
 #endif
-#endif /* not CANNOT_DUMP */
+#endif
 
   defsubr (&Skill_emacs);
 
+  defsubr (&Sinvocation_name);
+  defsubr (&Sinvocation_directory);
+
   DEFVAR_LISP ("command-line-args", &Vcommand_line_args,
     "Args passed by shell to Emacs, as a list of strings.");
 
   DEFVAR_LISP ("system-type", &Vsystem_type,
-    "Symbol indicating type of operating system you are using.");
+    "Value is symbol indicating type of operating system you are using.");
   Vsystem_type = intern (SYSTEM_TYPE);
 
+  DEFVAR_LISP ("system-configuration", &Vsystem_configuration,
+    "Value is string indicating configuration Emacs was built for.");
+  Vsystem_configuration = build_string (EMACS_CONFIGURATION);
+
   DEFVAR_BOOL ("noninteractive", &noninteractive1,
     "Non-nil means Emacs is running without interactive terminal.");
 
+  DEFVAR_LISP ("kill-emacs-hook", &Vkill_emacs_hook,
+    "Hook to be run whenever kill-emacs is called.\n\
+Since kill-emacs may be invoked when the terminal is disconnected (or\n\
+in other similar situations), functions placed on this hook should not\n\
+expect to be able to interact with the user.");
   Vkill_emacs_hook = Qnil;
 
-  DEFVAR_LISP ("kill-emacs-hook", &Vkill_emacs_hook,
-    "Function called, if non-nil, whenever kill-emacs is called.");
-}
+  DEFVAR_INT ("emacs-priority", &emacs_priority,
+    "Priority for Emacs to run at.\n\
+This value is effective only if set before Emacs is dumped,\n\
+and only if the Emacs executable is installed with setuid to permit\n\
+it to change priority.  (Emacs sets its uid back to the real uid.)\n\
+Currently, you need to define SET_EMACS_PRIORITY in `config.h'\n\
+before you compile Emacs, to enable the code for this feature.");
+  emacs_priority = 0;
 
+  DEFVAR_LISP ("invocation-name", &Vinvocation_name,
+    "The program name that was used to run Emacs.\n\
+Any directory names are omitted.");
+
+  DEFVAR_LISP ("invocation-directory", &Vinvocation_directory,
+    "The directory in which the Emacs executable was found, to run it.\n\
+The value is nil if that directory's name is not known.");
+
+  DEFVAR_LISP ("installation-directory", &Vinstallation_directory,
+    "A directory within which to look for the `lib-src' and `etc' directories.\n\
+This is non-nil when we can't find those directories in their standard\n\
+installed locations, but we can find them\n\
+near where the Emacs executable was found.");
+  Vinstallation_directory = Qnil;
+}
